package ALIGNMENT::default_align;
use strict;
require Exporter;
our $VERSION = '1.0';

use CLASSES::Protein;
use CLASSES::Fragment;
use CLASSES::Sequence_Alignment;
use TOOLS::assert;
use Bio::SearchIO;

our @ISA = qw(Exporter);
our @EXPORT = qw(default_align);

my $tool = "tfasty".TOOLS::constants::FASTA_VERSION;
my $params = "-E 0.001";
my $param_str = "Default parameters for tfasty34, with -E set to 0.001";
my $executable = TOOLS::constants::FASTA_DIR.$tool;

use constant ECUTOFF => 0.001;
use constant LENGTH_MIN => 0.60;

sub default_align {
    my ($protein, $fragment) = (shift,shift);
    my $ecutoff = shift;  $ecutoff = ECUTOFF unless defined $ecutoff;
    my $length_min = shift;   $length_min = LENGTH_MIN unless defined $length_min;
    assert($fragment && ref($fragment) eq "CLASSES::Fragment", "Bad fragment parameter");
    assert($protein && ref($protein) eq "CLASSES::Protein", "Bad protein parameter: $protein");
    assert($protein->get_dbh()==$fragment->get_dbh(), "Must be from the same database");
    my $dbh = $protein->get_dbh();

    my $lib_file = $protein->get_acc()."-".$fragment->get_acc().".tmp";
    open(OFILE, ">$lib_file");
    print OFILE ">".$fragment->get_acc()."\n".$fragment->get_sequence()."\n";
    close OFILE;

    my $query_string = ">".($protein->get_name() || "X")."\n".$protein->get_sequence();
    my $execute_string = "echo \"$query_string\" | $executable $params - $lib_file";

    my $fasta_handle;
    open($fasta_handle, "-|", $execute_string);

    my @alignments;
    my $in = new Bio::SearchIO(-format => 'fasta', -fh => $fasta_handle);

    my $A = undef;

    while (1) {
	my $result = $in->next_result() || last;
	my $hit = $result->next_hit() || last;
	my $hsp = $hit->next_hsp() || last;
	if ($hsp->evalue() > $ecutoff) {
		$A = "1";
		last;
	}
	if ($hsp->length('hit') < $length_min * $fragment->get_length()) {
		$A = "2";
		last;
	}
	last unless ($hsp->evalue() <= $ecutoff && $hsp->length('hit') >= $length_min * $fragment->get_length());
	my $protein_sequence = $hsp->query_string();
	$protein_sequence =~ s/\s//g;
	my $fragment_sequence = $hsp->hit_string();
	$fragment_sequence =~ s/\s//g;
	$A = CLASSES::Sequence_Alignment->new(dbh=>$dbh, protein_id=>$protein->get_id(), frag_id=>$fragment->get_id(), tool=>$tool, params=>$param_str, protein_start=>$hsp->start('query'), protein_finish=>$hsp->end('query'), fragment_start=>$hsp->start('hit'), fragment_finish=>$hsp->end('hit'), e_value=>$hsp->evalue(), aa_ident=>$hsp->frac_identical(), protein_align_seq=>$protein_sequence, frag_align_seq=>$fragment_sequence);
	last;
    }
	
    system("rm $lib_file");
    return $A;
}
