package CLASSES::Chromosome_Segment;
use strict;
use CLASSES::DBClass;
use CLASSES::Chromosome;
use TOOLS::assert;

our @VERSION = '1.0';
our @ISA = qw(CLASSES::DBClass);

#----------------------------
# Required methods
sub _init {
    return $_[0];
}

#---------------------------
# Other methods

###############
# Name: get_chromosome
# Purpose: (dynamic) Get the segments chromsome object
# Parameters: None
# Output: A chromosome object
sub get_chromosome {
    my $self = assert_dynamic(shift);

    return CLASSES::Chromosome->get_pk($self->get_dbh(), $self->get_chr_id());
}

###############
# Name: offset_to_absolute
# Purpose: (dynamic) Turn an offset into the segment to an index(s) on the chromosome
# Parameter: One or more coords
# Output: A list (or single) coord
sub offset_to_absolute {
    my $self = assert_dynamic(shift);
    my @offsets = @_;

    assert(@offsets > 0, "No coordinate given");
    assert(wantarray || @offsets==1, "Cannot invoke in scalar context with multiple parameters\n");

    my $start = $self->get_start();
    my $finish = $self->get_finish();
    my @results = map {assert($_ >= 0 && $_ <= $finish-$start, "Coordinate out of range: $_"); $_+$start} @offsets;
    return wantarray ? @results : $results[0];
}


###############
# Name: absolute_to_offset
# Purpose: (dynamic) Turn an index(s) on the chromosome to an offset on the segment
# Parameter: One or more coords
# Ouput A list (or single) coord
sub absolute_to_offset {
    my $self = assert_dynamic(shift);
    my @coords = @_;

    assert(@coords > 0, "No coordinate given");
    assert(wantarray || @coords==1, "Cannot invoke in scalar context with multiple parameters\n");

    my $start = $self->get_start();
    my $finish = $self->get_finish();
    my @results = map {assert($start <= $_ && $_ <= $finish, "Coordinate ot ouf range: $_\n"); $_-$start} @coords; 
    return wantarray ? @results : $results[0];
}

######################
# Name: get_size
# Purpose: (dynamic) Get the size of the segment
# Parameters: None
# Output: int
sub get_size {
    my $self = assert_dynamic(shift);
    return $self->get_finish() - $self->get_start() + 1;
}

######################
# Name: get_sequence
# Purpose: (dynamic) Print a susequence
# Parameters: Start offset (relative to + strand), finish_offset, strand (+/-, default +)
# Output: A string
sub get_sequence {
    my $self = assert_dynamic(shift);
    my ($start_offset, $finish_offset, $strand) = @_;
    assert($start_offset >=0 && $start_offset <= $finish_offset && $finish_offset <= $self->get_size(), "Bad offsets given");

    my $C = $self->get_chromosome();
    return $C->get_sequence($start_offset + $self->get_start(), $finish_offset + $self->get_start(), $strand);    
}
    


1;
