package CLASSES::DBClass_Versioned_Free;
use strict;
use CLASSES::DBClass_Versioned;
use QUERY::query_factory;
use QUERY::Where;
use TOOLS::assert;

our @VERSION = '1.0';
our @ISA = qw(CLASSES::DBClass_Versioned);

#######################
# Name: get_next_acc
# Purpose: Get the next free acc value
# Parameters: Database handle
# Output: An int
sub get_next_acc {
    my $class = assert_static(shift);
    my $dbh = assert_dbh(shift);

    $class =~ /(CLASSES\:\:)*(\w+)/;
    my $table = $2;
    my $query = $dbh->prepare("select max($table.acc) from $table");
    $query->execute();
    my $value = ($query->fetchrow())[0];

    return $value ? $value+1 : 1;
}

######################
# Name: insert
# Purpose: Augement CLASSES::DBClass_Versioned::insert to choose the new acc when undefined
# Parameters: No extra parmeters
# Output: None
# Side effects: Assignes new accs to any objects without accs
sub insert {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my @objs = ref($proto) ? ($proto) : @_;

    my $dbh = $objs[0]->get_dbh();
    my $next_acc = $class->get_next_acc($dbh);
    foreach (@objs) {
	assert($dbh = $_->get_dbh(), "All objects must have the same database handle\n");
	if ($_->get_acc()) {
	    assert($_->get_acc() < $next_acc, "User cannot assign object a new acc number");
	}
	else {
	    $_->set_acc($next_acc++);
	}
    }

    $class->CLASSES::DBClass_Versioned::insert(@objs);
}


######################
# Name: max
# Purpose: Return the max value of two numbers
# Input: Two numbers
# Output: The larger number
sub max {
    return $_[0] >= $_[1] ? $_[0] : $_[1];
}
