package CLASSES::Sequence_Alignment;
use strict;
use CLASSES::DBClass;
use TOOLS::assert;
use QUERY::query_factory;

our @VERSION = '1.0';
our @ISA = qw(CLASSES::DBClass);

################################################
# Required methods
sub _init {
    return $_[0];
}

################################################
# Other methods


######################
# Name: get_by_fragment
# Purpose: (static) Return all alignments involving a given fragment
# Parameters: a database handle, a fragment
# Output: A list of sequence alignment objects
sub get_by_fragment {
    my $class = assert_static(shift);
    my $dbh = assert_dbh(shift);
    my $frag = shift;
    assert(ref($frag) eq "CLASSES::Fragment", "Bad fragment object");

    my $where = "Sequence_Alignment.frag_id=?gbf_frag_id?";
    my @arr = CLASSES::Sequence_Alignment->get("dbh"=>$dbh,
					       "where"=>$where,
					       "saved_query"=>"sa_get_by_frag",
					       "values"=>{"gbf_frag_id"=>$frag->get_id()});
    return @arr;
}

######################
# Name: get_by_protein
# Purpose: (static) Return all alignments involving a given protein
# Parameters: a database handle, a protein
# Output: A list of sequence alignment objects
sub get_by_protein {
    my $class = assert_static(shift);
    my $dbh = assert_dbh(shift);
    my $protein = shift;
    assert(ref($protein) eq "CLASSES::Protein", "Bad protein object");

    my $where = "Sequence_Alignment.protein_id=?gbf_protein_id?";
    return CLASSES::Sequence_Alignment->get("dbh"=>$dbh,
					    "where"=>$where,
					    "saved_query"=>"sa_get_by_protein",
					    "values"=>{"gbf_protein_id"=>$protein->get_id()});
}

######################
# Name: get_by_fragment_and_protein
# Purpose: (static) Return all alignents of the given frament and protein
# Parameters: a database handle, a fragment object and a protein object
# Output: A sequence alignment object (or undef)
sub get_by_fragment_and_protein {
    my $class = assert_static(shift);
    my $dbh = assert_dbh(shift);
    my $fragment = shift;
    my $protein = shift;
    assert(ref($fragment) eq "CLASSES::Fragment", "Bad fragment object");
    assert(ref($protein) eq "CLASSES::Protein", "Bad protein object");

    my $where = "Sequence_Alignment.frag_id=?gbfp_frag_id? and Sequence_Alignment.protein_id=?gbfp_protein_id?";
    return CLASSES::Sequence_Alignment->get("dbh"=>$dbh,
					    "where"=>$where,
					    "saved_query"=>"sa_get_by_protein",
					    "values"=>{"gbfp_frag_id"=>$fragment->get_id(), "gbfp_protein_id"=>$protein->get_id()});
}

######################
# Name: get_protein
# Purpose: (dynamic) Get the associated protein object
# Parameters: None
# Output: A protein object
sub get_protein {
    my $self = assert_dynamic(shift);
    return CLASSES::Protein->get_pk($self->get_dbh(), $self->get_protein_id());
}

######################
# Name: get_fragment
# Purpose: (dynamic) Get the associated fragment object
# Parameters: None
# Output: A fragment object
sub get_fragment {
    my $self = assert_dynamic(shift);
    return CLASSES::Fragment->get_pk($self->get_dbh(), $self->get_frag_id());
}
