package QUERY::Query;
our $VERSION = '1.0';
use strict;

use QUERY::Where;
use TOOLS::assert;

sub new {
    my $class = assert_static(assert_abstract(shift));
    my %params = @_;

    my $self = bless {}, $class;

    # Parameters
    $self->{"_dbh"} = assert_dbh($params{"dbh"});
    $self->{"_command"} = $params{"command"};
    $self->{"_where"} = $params{"where"} || QUERY::Where->new();

    $self->{"_where"} = 
	!defined($params{"where"})              ? QUERY::Where->new() :
	ref($params{"where"}) eq "QUERY::Where" ? $params{"where"} :
     	                                          QUERY::Where->new($params{"where"});
					      
    # Other attributes
    $self->{"_vars"} = {};

    $self->_init(%params);
    $self->_prepare();

    $self->{"_num_positions"} = $self->count_num_positions();


    return $self;
}   

sub count_num_positions {
    my $self = shift;
    my $total = 0;
    for (values %{$self->{"_vars"}}) {
	$total += @$_;
    }
    return $total;
}

sub str {
    my $self = shift;
    return $self->{"_query"};
}

sub DESTROY {
    my $self = shift;
    $self->{"_sth"}->finish() if defined $self->{"_sth"};
}

sub _prepare {
    my $self = shift;
    $self->{"_sth"} = $self->{"_dbh"}->prepare($self->{"_query"});
}

sub execute {
    my $self = shift;
    my %values = @_ if defined $_[0];

    print $self->string_with_values(%values)."\n" if $values{"debug-print"};

    my %query_positions = %{$self->{"_vars"}};
    my %where_positions = %{$self->{"_where"}->get_variable_hash()};
    my $offset = $self->{"_num_positions"};
    my $sth = $self->{"_sth"};

    while (my ($variable, $value) = each %values) {
	my @positions;
	next if $variable =~ /^debug/;
	push(@positions, @{$query_positions{$variable}}) if exists $query_positions{$variable};
	push(@positions, map {$_+$offset} @{$where_positions{$variable}}) if exists $where_positions{$variable};
	assert(@positions > 0, "Bad variable: $variable\n");
	for (@positions) {
	    $sth->bind_param($_, $value);
	}
    }
    $sth->execute();
}

sub string_with_values {
    my $self = shift;
    my %variables = @_ if defined $_[0];

    my %query_positions = %{$self->{"_vars"}};
    my %where_positions = %{$self->{"_where"}->get_variable_hash()};
    my $offset = $self->{"_num_positions"};
    my $str = $self->{"_query"};

    my %H;
    while (my ($variable, $ref) = each %query_positions) {
	for (@$ref) {
	    $H{$_} = $variable;
	}
    }
    while (my ($variable, $ref) = each %where_positions) {
	for (@$ref) {
	    $H{$_+$offset} = $variable;
	}
    }

    for (my $i=1; $i <= keys %H; $i++) {
	my $val = defined $variables{$H{$i}} ? $variables{$H{$i}} : "NULL";
	$str =~ s/\?/\'$val\'/;
    }
    return $str;
}

1;
