package QUERY::Select;
our $VERSION = '1.0';
our @ISA = qw(QUERY::Query);
use strict;

use QUERY::Query;
use TOOLS::assert;

sub _init {
    my ($self, %params) = @_;

    # Parameters
    $self->{"_table_hash"} = $params{"table_hash"} or die("QUERY::Select::new: Need a table hash\n");
    $self->{"_function_fields"} = $params{"function_fields"} || $params{"functional_fields"};

    # Other variables
    $self->{"_op"} = "select";
    $self->{"_query"} = $self->_create_query();
    $self->{"_open"} = 0;
    return $self;
}


sub _create_query {
    my $self = shift;

    my $where = $self->{"_where"};
    my @all_atts;

    my $fun = $self->{"_function_fields"} || {};
    while (my ($table, $atts) = each %{$self->{"_table_hash"}}) {
	next unless $atts && @$atts;

	$table =~ s/^(.+\sas\s)//;
	my @arr = map {split(",", $_)} @$atts;
	foreach my $a (@arr) {
	    if (my $function = $fun->{"$table.$a"}) {
		push(@all_atts, "$function($table.$a)");
	    }
	    else {
		push(@all_atts, "$table.$a");
	    }
	}
    }
    @all_atts = ("*") unless @all_atts;

    my $query = $self->{"_op"}." ".join(",", @all_atts)." from ".join(",", keys %{$self->{"_table_hash"}});
    $query .= " where ".$where->create_clause() unless $where->empty();
    $query .= " ".$self->{"_command"} if $self->{"_command"};

    return $query;
}

sub execute {
    my $self = shift;
    my %vars = @_;

    $self->{"_open"} = 1;
    $self->QUERY::Query::execute(@_);
    print $self->explain_str(%vars)."\n" if $vars{"debug-explain"};
}

sub finish {
    my $self = shift;

    $self->{"_sth"}->finish();
    $self->{"_open"} = 0;
}

sub fetch_row {
    my $self = shift;

    ($self->{"_open"}) or die("QUERY::Select::fetch_row: Not open");
    return $self->{"_sth"}->fetchrow_hashref();
}

sub fetch_all {
    my $self = shift;

    ($self->{"_open"}) or die("QUERY::Select::fetch_all: Not open");
    my @results;

    while (my $ref = $self->fetch_row()) {
	push(@results, $ref);
    }

    $self->finish();

    return @results;
}

sub explain {
    my $self = shift;
    my %variables = @_ if defined $_[0];
    
    my $sth = $self->{"_dbh"}->prepare("explain ".$self->{"_query"});

    my %query_positions = %{$self->{"_vars"}};
    my %where_positions = %{$self->{"_where"}->get_variable_hash()};
    my $offset = $self->{"_num_positions"};

    while (my ($variable, $value) = each %variables) {
	my @positions;
	push(@positions, @{$query_positions{$variable}}) if exists $query_positions{$variable};
	push(@positions, map {$_+$offset} @{$where_positions{$variable}}) if exists $where_positions{$variable};
	assert(@positions > 0, "Bad variable: $variable\n");
	for (@positions) {
	    $sth->bind_param($_, $value);
	}
    }
    $sth->execute();

    my @arr;
    while (my $ref = $sth->fetchrow_arrayref()) {
	my @a = map {$_ ? $_ : "NULL"} @$ref;
	push(@arr, \@a);
    }

    $sth->finish();
    return @arr;
}

sub explain_str {
    my $self = shift;

    my @arr = $self->explain(@_);
    my $str = sprintf("%15s"x8, qw(table type possible_keys key key_len ref rows Extra));
    return "$str\n".join("\n", map {sprintf("%15s"x8,@$_)} @arr);
}
