package QUERY::Where;
our $VERSION = '1.0';
use strict;
use TOOLS::assert;

sub new {
    my $class = shift;
    my %params = 
	@_ == 0 ? ("clauses"=>[]) :
	@_ > 1  ? @_ :
	@_ == 1 && ref($_[0]) eq "ARRAY" ? ("clauses"=>$_[0]) : 
      	               ("clauses"=>[$_[0]]);

    my $self = bless {}, $class;

    if ($params{"unary_op"}) {
	!$params{"binary_op"} || die("Where::new: Can't have a unary operator AND a binary operator\n");
	$self->{"_unary_op"} = $params{"unary_op"} || "";    
    }
    else {
	$self->{"_binary_op"} = $params{"binary_op"} || "and";
    }

    $self->{"_clauses"} = [];
    $self->{"_vars"} = {};

    my $counter = 0;
    for my $clause (@{$params{"clauses"}}) {
	next unless defined $clause;
	unless (ref($clause)) {
	    my @arr = ($clause =~ /\?([\w_]+)\?/g);
	    $clause =~ s/\?[\w_]+\?/?/g;
	    push(@{$self->{"_clauses"}}, $clause);
	    for my $var (@arr) {
		$self->{"_vars"}->{$var} = [] unless exists $self->{"_vars"}->{"$var"};
		push(@{$self->{"_vars"}->{$var}}, ++$counter);
	    }
	}
	else {
	    assert(ref($clause) eq "QUERY::Where", "Bad clause: $clause\n");
	    next if $clause->empty();
	    push(@{$self->{"_clauses"}}, $clause);
	    my $h = $clause->get_variable_hash();
	    while (my ($var, $ref) = each %$h) {
		for (@$ref) {
		    $self->{"_vars"}->{$var} = [] unless exists $self->{"_vars"}->{"$var"};
		    push(@{$self->{"_vars"}->{$var}}, $_+$counter);
		}
	    }
	    $counter += $clause->num_positions();
	}
    }
    
    $self->{"_num_positions"} = $counter;
    ($self->{"_binary_op"} || @{$self->{"_clauses"}}==1) or die("Where::new: Uniary operator cannot have multiple clauses");
    return $self;
}

sub create_clause {
    my $self = shift;
    return "" if $self->empty();

    if ($self->{"_unary_op"}) {
	my $clause = $self->{"_clauses"}->[0];
	return $self->{"_unary_op"}." (".(ref($clause) ? $clause->create_clause() : $clause).")";
    }

    my $str = join(" ".$self->{"_binary_op"}." ", map {(ref($_) ? "(".$_->create_clause().")" : "($_)")} @{$self->{"_clauses"}});
    return "$str";
}

sub clause_with_vals {  # For debugging
    my $self = shift;
    my %params = @_;
    return "" if $self->empty();

    my %H;
    while (my ($var, $arr) = each %{$self->{"_vars"}}) {
        for my $pos (@$arr) {
	    $H{$pos} = $var;
        }
    } 

    my $str = $self->create_clause();
    for (my $counter = 1; $counter <= $self->num_positions(); $counter++) {
	my $val = $params{$H{$counter}} || "NULL";
	$str =~ s/^([^\?]*)\?/$1$val/;
    }
    return $str;
}

sub clause_with_vars {   # For debugging
    my $self = shift;
    return "" if $self->empty();

    my %H;
    while (my ($var, $arr) = each %{$self->{"_vars"}}) {
        for my $pos (@$arr) {
	    $H{$pos} = $var;
        }
    }

    my $str = $self->create_clause();
    for (my $counter = 1; $counter <= $self->num_positions(); $counter++) {
	my $var = $H{$counter} || next;
	$str =~ s/^([^\?]*)\?/$1\*\*\*$var\*\*\*/;
    }
    $str =~ s/\*\*\*/?/g;
    return $str;
}

sub get_variable_hash {
    my $self = shift;
    return $self->{"_vars"};
}

sub empty {
    my $self = shift;
    return @{$self->{"_clauses"}}==0;
}
	    
sub num_vars {
    my $self = shift;
    return scalar keys %{$self->{"_vars"}};
}

sub num_positions {
    my $self = shift;
    return $self->{"_num_positions"};
}

1;
