package UNION;

use strict;
use CLASSES::Pseudogene;
use CLASSES::PFSet;
use Data::Dumper;
use vars qw(@ISA @EXPORT @EXPORT_OK);

@ISA = qw(Exporter);
@EXPORT = qw(&overlap_two_PFSets &get_set_size &merge_ordered_lists &get_list_pgenes &overlap_two_lists &overlap &merge_set_accs &overlap_two_lists_backup &overlap_two_lists_histogram);
@EXPORT_OK = qw(&overlap_two_PFSets &get_set_size &merge_ordered_lists &get_list_pgenes &overlap_two_lists &overlap &merge_set_accs &overlap_two_lists_backup  &overlap_two_lists_histogram);

sub overlap_two_PFSets (){
	my ($dbh, $PFSet_id1, $PFSet_id2, $OVERLAP, $OVERLAP_RATIO) = @_;
		
	my $list1 = &get_list_pgenes($dbh, $PFSet_id1);
	my $list2 = &get_list_pgenes($dbh, $PFSet_id2);
	
	my ($new_list) = &overlap_two_lists($list1, $list2, $OVERLAP, $OVERLAP_RATIO);

	return $new_list; 
}

sub merge_set_accs(){
	# input is the ordered list of set accs(Generally started with the weighted set such as manually annotated set)
	# can be used to create union set of sets.

	my ($dbh, $set_acc_array, $OVERLAP, $OVERLAP_RATIO) = @_;

	my @set_array;
	for(my $i= 0; $i < @$set_acc_array; $i++){
		my $acc = $set_acc_array->[$i];
		my @PFSets = CLASSES::PFSet->get_by_acc($dbh, $acc);
		push @set_array, $PFSets[0]->get_id();
	}
	my $current = &merge_ordered_lists($dbh, \@set_array, $OVERLAP, $OVERLAP_RATIO);

	return $current;


}


sub merge_ordered_lists(){
	# input is the ordered list of set ids(Generally started with the weighted set such as manually annotated set)
	# can be used to create union set of sets.

	my ($dbh, $set_array, $OVERLAP, $OVERLAP_RATIO) = @_;


	my $current = &overlap_two_PFSets($dbh, $set_array->[0], $set_array->[1], $OVERLAP, $OVERLAP_RATIO);
	for (my $i= 2; $i < @$set_array; $i++ ){
		my $set_id = $set_array->[$i];
		my $pgene_list = &get_list_pgenes($dbh, $set_id);

		$current = &overlap_two_lists($current, $pgene_list, $OVERLAP, $OVERLAP_RATIO);
	}

	return $current;
}

sub get_list_pgenes(){
	my ($dbh, $set_id) = @_;

	my @PFSets = CLASSES::PFSet->get_by_id($dbh, $set_id);

	die "ERROR: one id correspond to more than one sets\n" if scalar(@PFSets) > 1;
	my @pgenes = $PFSets[0]->get_pgenes();

	return (\@pgenes); 
}

sub get_set_size(){
	my ($dbh, $set_acc) = @_;
	my @PFSets = CLASSES::PFSet->get_by_acc($dbh, $set_acc);
	my @pgenes = $PFSets[0]->get_pgenes();
	return scalar(@pgenes);
}

sub overlap_two_lists_histogram(){ 

	# the input has to be two lists of pgenes. 
	# the list1 will be the base list, list2 will be assessed based on list1
	# the program will calculate the overlap ratio for each pgene in list2

	my ($list1, $list2) = @_;

	#print "OVERLAP: LIST1:". scalar(@$list1). " LIST2:".scalar(@$list2)."\n";


	my $tax1 = $list1->[0]->tax_id();
	my $tax2 = $list2->[0]->tax_id();
	if ($tax1 != $tax2){
		die "ERROR: can't merge two sets from different species\n ";
	}

	my $build1 = $list1->[0]->get_chromosome()->get_build_id();
	my $build2 = $list2->[0]->get_chromosome()->get_build_id();
	die "ERROR: can't merge two sets from different Build\n" if $build1 != $build2;

	my %all_chrs;
	my %pgene_list1;
	my %pgene_list2;
	foreach my $pgene(@$list1){
		my $chr = $pgene->get_chromosome()->get_number();
		$all_chrs{$chr} = 1;
		my $strand = $pgene->get_strand();
		push @{$pgene_list1{$chr}{$strand}}, $pgene;
	}

	foreach my $pgene(@$list2){
		my $chr = $pgene->get_chromosome()->get_number();
		$all_chrs{$chr} = 1;
		my $strand = $pgene->get_strand();
		push @{$pgene_list2{$chr}{$strand}}, $pgene;
	}

	my %ratio;
	my %match;	
	foreach my $chr(keys %all_chrs){
		foreach my $strand("+", "-"){
			next if not defined $pgene_list2{$chr}{$strand} and not defined $pgene_list1{$chr}{$strand};
			if (not defined $pgene_list1{$chr}{$strand}){
				foreach my $pgene(@{$pgene_list2{$chr}{$strand}}){
					$ratio{$pgene->get_id()} = 0;
				}
				next;
			}

			next if not defined $pgene_list2{$chr}{$strand};

			my @plist1 = @{$pgene_list1{$chr}{$strand}};	
			my @plist2 = @{$pgene_list2{$chr}{$strand}};	

			foreach my $pgene2(@plist2){
				my ($s2, $f2) =$pgene2->get_coords();
				my $max = 0;
				foreach my $pgene1(@plist1){
					my ($s1, $f1) = $pgene1->get_coords();
					my $overlap = &overlap_ratio($s2, $f2, $s1, $f1);	
					if ($overlap > $max){
						$max = $overlap;
						$match{$pgene2->get_id()} = $pgene1->get_id();
					}
				}
				$ratio{$pgene2->get_id()} = $max;
			}
		}
	}
	return (\%ratio, \%match);
}

sub overlap_two_lists_backup(){ 

	# the input has to be two lists of pgenes. 
	# the list1 will be the base list, list2 will be assessed based on list1

	my ($list1, $list2, $OVERLAP, $OVERLAP_RATIO) = @_;

	#print "OVERLAP: LIST1:". scalar(@$list1). " LIST2:".scalar(@$list2)."\n";


	my $tax1 = $list1->[0]->tax_id();
	my $tax2 = $list2->[0]->tax_id();
	if ($tax1 != $tax2){
		die "ERROR: can't merge two sets from different species\n ";
	}

	my $build1 = $list1->[0]->get_chromosome()->get_build_id();
	my $build2 = $list2->[0]->get_chromosome()->get_build_id();
	die "ERROR: can't merge two sets from different Build\n" if $build1 != $build2;

	my %all_chrs;
	my %pgene_list1;
	my %pgene_list2;
	foreach my $pgene(@$list1){
		my $chr = $pgene->get_chromosome()->get_number();
		$all_chrs{$chr} = 1;
		my $strand = $pgene->get_strand();
		push @{$pgene_list1{$chr}{$strand}}, $pgene;
	}

	foreach my $pgene(@$list2){
		my $chr = $pgene->get_chromosome()->get_number();
		$all_chrs{$chr} = 1;
		my $strand = $pgene->get_strand();
		push @{$pgene_list2{$chr}{$strand}}, $pgene;
	}

	my @final_list;
	my $final_count = 0;
	foreach my $chr(keys %all_chrs){
		foreach my $strand("+", "-"){
			next if not defined $pgene_list2{$chr}{$strand} and not defined $pgene_list1{$chr}{$strand};
			if (not defined $pgene_list1{$chr}{$strand}){
				@final_list = (@final_list, @{$pgene_list2{$chr}{$strand}});
				next;
			}

			if (not defined $pgene_list2{$chr}{$strand}){
				@final_list = (@final_list, @{$pgene_list1{$chr}{$strand}});
				next;
			}

			my @plist1 = @{$pgene_list1{$chr}{$strand}};	
			my @plist2 = @{$pgene_list2{$chr}{$strand}};	

			my %final;
			foreach my $pgene2(@plist2){
				my ($s2, $f2) =$pgene2->get_coords();
				my $decide = 0;
				my $count = 0;
				foreach my $pgene1(@plist1){
					my ($s1, $f1) = $pgene1->get_coords();
					my $overlap = &overlap($s2, $f2, $s1, $f1, $OVERLAP, $OVERLAP_RATIO);	
					$final{$pgene1->get_id()} = 1 if $overlap == 1;
					$count++ if $overlap == 1;
					last if $overlap;
				}
				push @final_list, $pgene2;
				if ($count > 3){
					#print "ID:".$pgene2->get_id()." remove $count pgenes in big list\n"; 
				}
			}

			$final_count += scalar(keys %final);
			foreach my $thing(@plist1){
				next if defined $final{$thing->get_id()};
				push @final_list, $thing;
			}
		}
	}
	#print "IN TOTAL: $final_count has been overlpaped out\n";
	return (\@final_list);
}

sub overlap_two_lists(){ 

	# the input has to be two lists of pgenes. 
	# the list1 will be the base list, list2 will be assessed based on list1

	my ($list1, $list2, $OVERLAP, $OVERLAP_RATIO) = @_;

	#print "OVERLAP: LIST1:". scalar(@$list1). " LIST2:".scalar(@$list2)."\n";


	my $tax1 = $list1->[0]->tax_id();
	my $tax2 = $list2->[0]->tax_id();
	if ($tax1 != $tax2){
		die "ERROR: can't merge two sets from different species\n ";
	}

	my $build1 = $list1->[0]->get_chromosome()->get_build_id();
	my $build2 = $list2->[0]->get_chromosome()->get_build_id();
	die "ERROR: can't merge two sets from different Build\n" if $build1 != $build2;

	my %all_chrs;
	my %pgene_list1;
	my %pgene_list2;
	foreach my $pgene(@$list1){
		my $chr = $pgene->get_chromosome()->get_number();
		$all_chrs{$chr} = 1;
		my $strand = $pgene->get_strand();
		push @{$pgene_list1{$chr}{$strand}}, $pgene;
	}

	foreach my $pgene(@$list2){
		my $chr = $pgene->get_chromosome()->get_number();
		$all_chrs{$chr} = 1;
		my $strand = $pgene->get_strand();
		push @{$pgene_list2{$chr}{$strand}}, $pgene;
	}

	my @final_list;
	foreach my $chr(keys %all_chrs){
		foreach my $strand("+", "-"){
			next if not defined $pgene_list2{$chr}{$strand} and not defined $pgene_list1{$chr}{$strand};
			if (not defined $pgene_list1{$chr}{$strand}){
				@final_list = (@final_list, @{$pgene_list2{$chr}{$strand}});
				next;
			}

			if (not defined $pgene_list2{$chr}{$strand}){
				@final_list = (@final_list, @{$pgene_list1{$chr}{$strand}});
				next;
			}

			my @plist1 = @{$pgene_list1{$chr}{$strand}};	
			my @plist2 = @{$pgene_list2{$chr}{$strand}};	

			foreach my $pgene2(@plist2){
				my ($s2, $f2) =$pgene2->get_coords();
				my $decide = 0;
				foreach my $pgene1(@plist1){
					my ($s1, $f1) = $pgene1->get_coords();
					my $overlap = &overlap($s2, $f2, $s1, $f1, $OVERLAP, $OVERLAP_RATIO);	
					$decide = $overlap;
					last if $overlap;
				}
				next if $decide;	
				push @plist1, $pgene2;	
			}
			@final_list = (@final_list, @plist1);
		}
	}
	#print "Final List:".scalar(@final_list)."\n";
	return (\@final_list);
}

sub overlap(){
        #return true if sequence have overlap greater than minimum domain size;
        #return 1 if they overlap, 0 if not;

        my ($A,$B,$C,$D, $MIN_OVERLAP, $MIN_OVERLAP_RATIO) = @_;
        #$A $B are the start and stop for seq_ac1, $C and $D are for seq_ac2

        #zero overlap cases
        if ($C >= $B or $A >= $D ) { return 0; }

        my $shortest = ($B-$A) > ($D-$C) ? ($D-$C) : ($B-$A);
        my $start = $A <= $C ? $C : $A;
        my $stop = $B <= $D ? $B : $D;

        my $overlap = $stop - $start;
        my $overlap_ratio = $overlap/$shortest;

#       print "A-B:[$A-$B] C-D:[$C-$D] $overlap [ $overlap_ratio ]\n";
        if ($overlap > $MIN_OVERLAP and $overlap_ratio > $MIN_OVERLAP_RATIO ) {
                return 1;
        } else {
                return 0; #too small to be considered and overlap;
        }
}

sub overlap_ratio(){

        my ($A,$B,$C,$D) = @_;
        #$A $B are the start and stop for seq_ac1, $C and $D are for seq_ac2

        #zero overlap cases
        if ($C >= $B or $A >= $D ) { return 0; }

        my $shortest = ($B-$A) > ($D-$C) ? ($D-$C) : ($B-$A);
        my $start = $A <= $C ? $C : $A;
        my $stop = $B <= $D ? $B : $D;

        my $overlap = $stop - $start;
        my $overlap_ratio = $overlap/$shortest;

	return $overlap_ratio;
}

1;
