#!/usr/bin/perl -w
use strict;

use CLASSES::Protein;
use CLASSES::Pseudogene;
use SEARCH::Search;
use DB::db;

$| = 1;

my @sources = ("pseudogene.org", "Bork", "Hoppsigen");
my @types = ("Ambiguous", "Duplicated", "Non-processed", "Processed");

#my ($value, $field) = (shift, shift);
my $table = shift || "Pseudogene";
my $tax_id = shift || 9606;
my $chr = shift;
$chr =~ s/^chr// if $chr;

my $dbh = DB::db::getDB("db" => "Pgenes_$tax_id");
my $B = CLASSES::Build->current_build($dbh);

my @pgenes = $chr ? CLASSES::Pseudogene->get_by_location($dbh, $B->get_chromosome($chr)) : CLASSES::Pseudogene->get(dbh=>$dbh);

# Calculate total contents
print "Total contents: ".@pgenes."\n";

# Split out databases
my %H;
for my $P (@pgenes) {
    $H{$P->get_source()} = [] unless exists $H{$P->get_source()};
    push(@{$H{$P->get_source}}, $P);
}

foreach my $source (@sources) {
    @{$H{$source}} = sort cmp_pgenes @{$H{$source}};
}

foreach my $source (@sources) {
    print "$source: ".scalar(@{$H{$source}})."\n";
    for my $type (@types) {
	print "     $type: ".scalar(grep {$_->get_ptype() eq $type} @{$H{$source}})."\n";
    }
    for my $source2 (@sources) {
	next if $source eq $source2;
	print "     $source/$source2: ".join(" / ", intersection($H{$source}, $H{$source2}))."\n";

	my @P1 = grep {$_->get_ptype() eq 'Processed'} @{$H{$source}};
	my @P2 = grep {$_->get_ptype() eq 'Processed'} @{$H{$source2}};
	print "     $source/$source2 (processed): ".join(" / ", intersection(\@P1, \@P2))."\n";
    }
}

my @P0 = grep {$_->get_ptype() eq 'Processed'} @{$H{"Hoppsigen"}};
my @P1 = grep {$_->get_ptype() eq 'Processed'} @{$H{"pseudogene.org"}};
my @P2 = grep {$_->get_ptype() eq 'Non-processed'} @{$H{"Bork"}};
print "PgO(P)/Bork(D): ".join(" / ", intersection(\@P1, \@P2))."\n";
print "Bork(D)/PgO(P): ".join(" / ", intersection(\@P2, \@P1))."\n";
print "Bork(D)/Hop(P): ".join(" / ", intersection(\@P2, \@P0))."\n";
print "Hop(P)/Bork(D): ".join(" / ", intersection(\@P0, \@P2))."\n";

@P1 = grep {$_->get_ptype() eq 'Duplicated'} @{$H{"pseudogene.org"}};
@P2 = grep {$_->get_ptype() eq 'Processed'} @{$H{"Bork"}};
print "PgO(D)/Bork(P): ".join(" / ", intersection(\@P1, \@P2))."\n";
print "Bork(P)/PgO(D): ".join(" / ", intersection(\@P2, \@P1))."\n";
print "PgO(D)/Hop(P): ".join(" / ", intersection(\@P1, \@P0))."\n";
print "Hop(P)/PgO(D): ".join(" / ", intersection(\@P0, \@P1))."\n";

sub intersection {
    my @arr1 = @{$_[0]};
    my @arr2 = @{$_[1]};

    my ($counter_intersect, $counter_unique) = (0,0);
    my ($i, $j) = (0,0);
    while ($i < @arr1 && $j < @arr2) {
	while ($j < @arr2 && pgene_lt($arr2[$j], $arr1[$i])) {
	    $j++;
	}

	if ($j < @arr2 && ($arr1[$i]->get_chromosome()->get_number() eq $arr2[$j]->get_chromosome()->get_number() && ($arr1[$i]->get_coords())[1] >= ($arr2[$j]->get_coords())[0])) {
	    $counter_intersect++;
	}
	else {
	    $counter_unique++;
	}
	$i++;
    }
    $counter_unique++ && $i++ while ($i < @arr1);

    return ($counter_intersect, $counter_unique);
}

sub cmp_pgenes {
    return $a->get_chromosome()->get_number() cmp $b->get_chromosome()->get_number() ||
	($a->get_coords())[0] <=>($b-> get_coords())[0] ||
	($a->get_coords())[1] <=>($b-> get_coords())[1];
}

sub pgene_lt {
    return 1 if ($_[0]->get_chromosome()->get_number() lt $_[1]->get_chromosome()->get_number());
    return 0 if ($_[0]->get_chromosome()->get_number() gt $_[1]->get_chromosome()->get_number());
    return ($_[0]->get_coords)[1] < ($_[1]->get_coords)[0];
}
