#!/usr/bin/perl -w
use strict;

use CLASSES::Protein;
use CLASSES::Pseudogene;
use SEARCH::Search;
use DB::db;

$| = 1;

#my ($value, $field) = (shift, shift);
my $table = shift || "Pseudogene";
my $tax_id = shift || 9606;

my $dbh = DB::db::getDB("db" => "Pgenes_$tax_id");

my $str = "CLASSES\:\:$table";

my %H = ("source" => ["pseudogene.org", "Bork", "Hoppsigen"]);
my $B = CLASSES::Build->current_build($dbh);
my $C = $B->get_chromosome(22);

for my $field (keys %H) {
    for my $value (@{$H{$field}}) {
	for my $value2 (@{$H{$field}}) {
	    next if $value eq $value2;
	    my @arr1 = $str->get_by_location($dbh, $C, undef, undef, where => "$field = '$value'");
	    my @arr2 = $str->get_by_location($dbh, $C, undef, undef, where => "$field = '$value2'");
	    print "$value v. $value2: ".join(", ", intersection(\@arr1, \@arr2))."\n";
	    for my $type ('d', 'p', 'n') {
		my @arr3 = grep {$_->get_ptype() eq $type} @arr1;
		my @arr4 = grep {$_->get_ptype() eq $type} @arr2;
		print "   $type: ".join(", ", intersection(\@arr3, \@arr4))."\n";
	    }
	}
    }
}

sub intersection {
    my @arr1 = sort cmp_pgenes @{$_[0]};
    my @arr2 = sort cmp_pgenes @{$_[1]};

    my ($counter_intersect, $counter_unique) = (0,0);
    my ($i, $j) = (0,0);
    while ($i < @arr1 && $j < @arr2) {
	while ($j < @arr2 && pgene_lt($arr2[$j], $arr1[$i])) {
	    $j++;
	}

	if ($j < @arr2 && ($arr1[$i]->get_chromosome()->get_number() eq $arr2[$j]->get_chromosome()->get_number() && ($arr1[$i]->get_coords())[1] >= ($arr2[$j]->get_coords())[0])) {
	    $counter_intersect++;
	}
	else {
	    print "Unique: ".join(", ", $arr1[$i]->get_coords())."   ".join(", ", $arr2[$j]->get_coords())."\n";
	    $counter_unique++;
	}
	$i++;
    }

    return ($counter_intersect, $counter_unique);
}

    

sub max {
    return $_[0] < $_[1] ? $_[1] : $_[0];
}

sub overlapping_pgenes {
    my ($p1, $p2) = @_;

    return 0 unless $p1->get_chromosome() == $p2->get_chromosome();

    my @c1 = $p1->get_coords();
    my @c2 = $p2->get_coords();

    if ($c1[0] <= $c2[0]) {
	return $c1[1] >= $c2[0];
    }
  
    return $c2[1] >= $c1[1];
}

# Returns 1 of first pgene ends before second one starts
sub pgene_lt {
    return 1 if ($_[0]->get_chromosome()->get_number() lt $_[1]->get_chromosome()->get_number());
    return 0 if ($_[0]->get_chromosome()->get_number() gt $_[1]->get_chromosome()->get_number());
    return ($_[0]->get_coords)[1] < ($_[1]->get_coords)[0];
}

sub cmp_pgenes {
    return $a->get_chromosome()->get_number() cmp $b->get_chromosome()->get_number() ||
	($a->get_coords())[0] <=>($b-> get_coords())[0] ||
	($a->get_coords())[1] <=>($b-> get_coords())[1];
}
