#!/usr/bin/perl -w
use strict;

use DB::db;
use TaxID::tax_id;
use CLASSES::Pseudogene;
use QUERY::query_factory;
use CLASSES::PFSet;

my %H = get_tax_id_hash(1);
my @tax_ids = keys %H;

my $prok_total = 0;
my $euk_total = 0;
for my $tax_id (@tax_ids) {
    my $dbh = DB::db::getDB("db" => "Pgenes_$tax_id");

    print "Organism: ".$H{$tax_id}."\n";
    if (get_type($tax_id) eq 'Eukaryote') {
	my @S = CLASSES::PFSet->get(dbh=>$dbh);
	for my $S (@S) {
	    my @P = $S->get_pgenes();
	    print "   ".$S->get_name().": ".(scalar(@P));
	    my @arr = count_types(\@P);
	    print "   ".join(", ", (@arr[0..3], $arr[1]+$arr[2]))."\n";
	}

	my @P = CLASSES::Pseudogene->get(dbh=>$dbh);
	print "   Total: ".scalar(@P)."\n";
	$euk_total += @P;
    }
    else {
	my @arr = CLASSES::Pseudogene->get(dbh=>$dbh);
	$prok_total+= @arr;
    }
    print "\n";
}


print "Totals:\n";
print "   Euk: $euk_total\n   Prok: $prok_total\n";

sub count_types {
    my %types;
    $types{Processed} = 0;
    $types{"Non-processed"} = 0;
    $types{Ambiguous} = 0;
    $types{Duplicated} = 0;
    map {$types{$_->get_ptype()}=0 unless $types{$_->get_ptype()};  $types{$_->get_ptype()} = $types{$_->get_ptype()}+1;} @{$_[0]};
    return map {$types{$_}} qw(Processed Non-processed Duplicated Ambiguous);
} 
