#!/usr/bin/perl -w
use strict;

use CLASSES::Protein;
use CLASSES::Pseudogene;
use SEARCH::Search;
use DB::db;

my $min_ratio = 0.80;
my $dbh = DB::db::getDB("db" => "Pgenes_9606");

my $chr = shift;

my @sources = ('pseudogene.org', 'Bork', 'Hoppsigen');

my $B = CLASSES::Build->current_build($dbh);
my @pgenes = $chr ? CLASSES::Pseudogene->get_by_location($dbh, $B->get_chromosome($chr)) : CLASSES::Pseudogene->get(dbh=>$dbh);

my %H;
for my $source (@sources) {
    $H{$source} = [sort(cmp_pgenes grep {$_->get_source() eq $source && $_->get_ptype() eq 'Processed'} @pgenes)];
}

my @sets = ();
for (my $i=0; $i < @sources; $i++) {
    for my $P (@{$H{$sources[$i]}}) {
	my $total = 0;
	for my $s (@sets) {
	    if (grep {same_pgene($P, $_)} @$s) {
		push(@$s, $P);
		$total++;
	    }
	}
	push(@sets, [$P]) unless $total;
    }
}

my @H2;
my @c;
for my $s (@sets) {
    my $code = 0;
    my $shift = 1;
    for my $source (@sources) {
	$code = $code | $shift if grep {$_->get_source() eq $source} @$s;
	$shift = $shift << 1;
    }
    $H2[$code] = [] unless defined $H2[$code];
    push(@{$H2[$code]}, $s);
}

print join(", ", @sources)."\n";
for (my $i=0; $i < 8; $i++) {
    print "$i: ".($H2[$i] ? @{$H2[$i]} : 0)."\n";
}

sub search_sets {
    my @arr = @{$_[0]};
    my $P = $_;
    my $chr = $P->get_chromosome()->get_number();
    my @coords = $P->get_coords();

    my ($a, $b) = (0, @arr-1);
    while ($a < $b-1) {
	my $mid = ($a+$b)/2;
	if (grep {$_->get_chromosome()->get_number() lt $chr || ($_-get_chromosome()->get_number() eq $chr && ($_->get_coords())[1] < $coords[0]} < 0} @arr[$mid]) {
	    $b = $mid-1;
	}
	else {
	    $a = $mid;
	}
    }

    return max($mid-1, 0);
}

sub same_pgene {
    return 0 unless $_[0]->get_chromosome()->get_number eq $_[1]->get_chromosome()->get_number() && $_[0]->get_strand eq $_[1]->get_strand();

    my @c1 = $_[0]->get_coords();
    my @c2 = $_[1]->get_coords();

    my $overlap = &min($c2[1]-$c1[0], $c1[1]-$c2[0]);
    return $overlap > $min_ratio*&max($c1[1]-$c1[0], $c2[1]-$c2[0]);
}

sub cmp_pgenes {
    return $a->get_chromosome()->get_number() cmp $b->get_chromosome()->get_number() ||
	($a->get_coords())[0] <=>($b-> get_coords())[0] ||
	($a->get_coords())[1] <=>($b-> get_coords())[1];
}

sub min {
    return $_[0] <= $_[1] ? $_[0] : $_[1];
}

sub max {
    return $_[0] >= $_[1] ? $_[0] : $_[1];
}

