#!/usr/bin/perl -w
use strict;

use CLASSES::Protein;
use CLASSES::Pseudogene;
use SEARCH::Search;
use DB::db;

my $min_ratio = 0.90;
my $dbh = DB::db::getDB("db" => "Pgenes_9606");

my @sources = ("pseudogene.org", "Bork", "Hoppsigen");

my $chr = shift;

my $B = CLASSES::Build->current_build($dbh);
my @pgenes = sort cmp_pgenes $chr ? CLASSES::Pseudogene->get_by_location($dbh, $B->get_chromosome($chr), undef) : CLASSES::Pseudogene->get(dbh=>$dbh, where=>"ptype = 'Processed'");

my @groups = ();
for (my $i=1; $i < @pgenes; $i++) {
    if (!same_pgene($pgenes[$i], $pgenes[$i-1])) {
	push(@groups, $i);
    }
}
push(@groups, scalar @pgenes);

my @count = (0,0,0,0,0,0,0,0,0);
for (my $i=0; $i < @groups-1; $i++) {
    my @list = @pgenes[$groups[$i]..($groups[$i+1]-1)];
    my $bit = 1;
    my $code = 0;
    for my $s (@sources) {
	$code = $code | $bit if (grep {$_->get_source() eq $s} @list) > 0;
	$bit = 2*$bit;
    }
    $count[$code]++;
}

print join(", ", @sources)."\n";
for (my $i=1; $i < 8; $i++) {
    print "$i: ".$count[$i]."\n";
}


sub same_pgene {
    return 0 unless $_[0]->get_chromosome()->get_number eq $_[1]->get_chromosome()->get_number() && $_[0]->get_strand eq $_[1]->get_strand();

    my @c1 = $_[0]->get_coords();
    my @c2 = $_[1]->get_coords();

    my ($P1, $P2) = map {$_->get_protein()} (@_);
    my ($acc1, $acc2) = map {$_ && ref($_) eq "CLASSES::Protein" ? $_->get_acc() : undef} ($P1, $P2);

    my $overlap = &min($c2[1]-$c1[0], $c1[1]-$c2[0]);
    return 1 if $overlap > 0 && $acc1 && $acc2 && $acc1 eq $acc2;
    return $overlap > $min_ratio*&max($c1[1]-$c1[0], $c2[1]-$c2[0]);
}

sub cmp_pgenes {
    return $a->get_chromosome()->get_number() cmp $b->get_chromosome()->get_number() ||
	($a->get_coords())[0] <=>($b-> get_coords())[0] ||
	($a->get_coords())[1] <=>($b-> get_coords())[1];
}

sub min {
    return $_[0] <= $_[1] ? $_[0] : $_[1];
}

sub max {
    return $_[0] >= $_[1] ? $_[0] : $_[1];
}

