#!/usr/bin/perl -w
use strict;

use CLASSES::Protein;
use CLASSES::Pseudogene;
use SEARCH::Search;
use DB::db;

my $min_ratio = shift(@ARGV);
my $dbh = DB::db::getDB("db" => "Pgenes_9606");
my $chr = 22;
my $ptype = undef; #'Processed';

my $B = CLASSES::Build->current_build($dbh);
my $C = $chr ? $B->get_chromosome($chr) : undef;

my @set_accs = @ARGV;

# Get all the pseudogenes from the sets
# H will be a hash mapping of every pseudogene in a set to all set ids containing that pseudogene
my @sets;
my %id_hash;

for (@set_accs) {
    push(@sets, CLASSES::PFSet->get_by_acc($dbh, $_));
}

my $bit = 1;
for (my $i=0; $i < @sets; $i++) {
    $id_hash{$sets[$i]->get_id()} = $bit;
    $bit = $bit << 1;
}

my %H;
for my $S (@sets) {
    my @arr = grep {(!$C || $_->get_chromosome() == $C) && (!$ptype || $_->get_ptype eq $ptype)} $S->get_pgenes();
#    if ($S->get_acc()==3) {
#	@arr = grep {$_->get_source() =~ 'pseudogene.org'} @arr;
#    }
    print $S->get_acc().": ".scalar(@arr)."\n";
    for my $P (@arr) {
	$H{$P->get_id()} = [] unless defined $H{$P->get_id()};
	push(@{$H{$P->get_id()}}, $S->get_id());
    }
}

# Create the list of pseudogenes
my @pgenes = sort cmp_pgenes map {CLASSES::Pseudogene->get_pk($dbh, $_)} keys %H;
print "Total: ".scalar(@pgenes)."\n";

# "Group" adjacent pseudogenes so they can be "merged" if they are the "same"
my @groups = (0);
for (my $i=1; $i < @pgenes; $i++) {
    if (!same_pgene($pgenes[$i], $pgenes[$i-1])) {
	push(@groups, $i);
    }
}
push(@groups, scalar @pgenes);

my @count;
my $i;
for (my $i=0; $i < @groups-1; $i++) {
    my $code = 0;
    for (my $j=$groups[$i]; $j < $groups[$i+1]; $j++) {
	my $Pgene = $pgenes[$j];
	for my $id (@{$H{$Pgene->get_id()}}) {
	    $code = $code | $id_hash{$id};
	}
    }
    $count[$code]++;
}

for (my $code=1; $code < @count; $code++) {
    if ($count[$code]) {
	my $p = 1;
	for (my $j=0; $j < @sets; $j++) {
	    if ($code & $p) {
		print $sets[$j]->get_acc()." ";
	    }
	    $p <<= 1;
	}
	print ": ".$count[$code]."\n";
    }
}


sub same_pgene {
    return 0 unless $_[0]->get_chromosome()->get_number() eq $_[1]->get_chromosome()->get_number() && $_[0]->get_strand eq $_[1]->get_strand();

    my @c1 = $_[0]->get_coords();
    my @c2 = $_[1]->get_coords();

    my ($P1, $P2) = map {$_->get_protein()} (@_);
    my ($acc1, $acc2) = map {$_ && ref($_) eq "CLASSES::Protein" ? $_->get_acc() : undef} ($P1, $P2);

    my $overlap = &min($c2[1]-$c1[0], $c1[1]-$c2[0]);
    return 1 if $overlap > 0 && $acc1 && $acc2 && ($acc1 eq $acc2);
    return $overlap > $min_ratio*&max($c1[1]-$c1[0], $c2[1]-$c2[0]);
}

sub cmp_pgenes {
    return $a->get_chromosome()->get_number() cmp $b->get_chromosome()->get_number() ||
	($a->get_coords())[0] <=>($b-> get_coords())[0] ||
	($a->get_coords())[1] <=>($b-> get_coords())[1];
}

sub min {
    return $_[0] <= $_[1] ? $_[0] : $_[1];
}

sub max {
    return $_[0] >= $_[1] ? $_[0] : $_[1];
}

