#!/usr/bin/perl -w
use strict;

use DBI;
use CGI qw(:all);
use CLASSES::Build;
use CLASSES::Chromosome;
use CLASSES::Pseudogene;
use DB::db;


print header();

my $tax_id = param("tax_id") || 9606;
my $org = param("org") || 'Human';
my $chr_num = param("ch") || 1;
my $method = param("method") || 'View whole chromosome';
my $start = param("start") || 2000000;
my $stop = param("end") || 30000000;

my $dbh = DB::db::getDB(db => "Pgenes_$tax_id");

my $B = CLASSES::Build->current_build($dbh);
my @results;

if ($method eq 'View whole chromosome') {    # Get all pgenes on the chromosome
    $start = 1;
    $stop = undef;
    print "<FONT SIZE='4' span class='h1'>Pseudogenes in chromosome $chr_num ($org): </span>";

}

my $C = $B->get_chromosome($chr_num);
$start = $start || 1;
@results = CLASSES::Pseudogene->get_by_location($dbh, $C, $start, $stop);
print "<FONT SIZE='4' span class='h1'>Pseudogenes in chromosome $chr_num ($org) from $start to $stop:</span>" unless $method eq 'View whole chromosome';

print "<html>";

my $length = @results;

#display results
if ($length == 0 ) {
    print "<FONT SIZE='4pt' CLASS='san-serif' FACE='arial, Helvetica'>";
    print "<p>There are no pseudogenes in this region!";
    print "<p><center><A href='./pseudo.cgi'>Go back</A><p>";
}


else {
            print "<FONT SIZE='-1' color ='green' CLASS='sans-serif' FACE='arial, Helvetica'>",
    "<P><IMG border ='0' src = '../human/reddot.gif'>Click on the red button to view nucleotide sequence.",
    "<P><IMG border ='0' src = '../human/yellowdot.gif'>Click on the yellow button to view amino acid sequence.</font>",
    
    "<FONT SIZE='3pt' color='blue' CLASS='sans-serif' FACE='arial, Helvetica'><P>Click on the Pgene ACC to view the corresponding pseudogene annotation.</p>",
#	    "<P><A href='chrom-text.cgi?tax_id=&tax_id&org=$org&ch=$chr_num&method=$method&start=$start&stop=$stop'>Click here to get the information in text format</A></P>",
	    "</FONT></center><hr>";	
    my @attr = ("Links", 
	    "Pseudogene Acc", "Choromosome", "Chromsome Start/End", "Protein", "Query Start/End", "E-value", "Amino Acid Identity", "Polya", "Disable"); 
    
    print "<FONT SIZE='-1' color ='green' CLASS='sans-serif' FACE='arial, Helvetica'>",
    #"<P><IMG border ='0' src = '../human/reddot.gif'>Click on the red button to view nucleotide sequence and its alignment to CDS sequence.",
    #"<P><IMG border ='0' src = '../human/yellowdot.gif'>Click on the yellow button to view predicted amino acid sequence and alignment to functional protein sequence.",
    "</FONT><hr>",
    "<TABLE align='center' CELLPADDING='2' WIDTH='1200' CELLSPACING='1' BORDER='1'>",
    "<TR align='center' BGCOLOR='#e3fdfa'>";
    foreach my $at (@attr) {
	print "<TD><FONT CLASS='sans-serif' FACE='arial, Helvetica'><B>$at</B></FONT></TD>",
    }
    
    for my $P (@results) {
	my $id = $P->get_id();
	my $acc = $P->get_acc();
	my $name = $P->pgene_name();
	my $Protein = $P->get_protein();
	my $p_acc = $Protein->get_acc();
	my $p_name = $Protein->get_name();
	my $chr_num = $P->get_chromosome()->get_number();
	my ($start, $stop) = $P->get_coords();
	my $strand = $P->get_strand;
	my $type = ($P->get_ptype() eq 'p') ? 'Processed' : 'Duplicated';
	my @A = $P->get_alignments();
	my ($p_start, $p_stop) = ($A[0]->get_protein_start(), $A[-1]->get_protein_finish());
	my $e_value = (@A==1) ? ($A[0]->get_e_value() || "NULL") : "NA";
	my $aa_identity = (@A == 1) ? ($A[0]->get_aa_ident() || "NULL") : "NA";
	$aa_identity = sprintf("%.2f", $aa_identity);
	my $polya = $P->get_polya();
	my $disable = $P->get_disable();
	
	my $aa = $P->get_aligned_translation();
	my $dna = $P->get_sequence();
	
	print "<TR align='center' BGCOLOR='#F3F3F3'>";
	print "<TD><A href='dna_pr_display.cgi?c_type=nucleotide&acc=$acc&seq=$dna' target='_new'>",
	"<img border='0' src='../human/reddot.gif'></A>",
	"<A href='dna_pr_display.cgi?c_type=amino_acid&acc=$acc&seq=$aa' target='_new'>", 
	"<img width='12' border='0' src='../human/yellowdot.gif'></A></TD>";
#	print "<TD><A href='alignment.cgi?gid=$gid&class=$class_new&seq=dna' target='_new'>",
#	"<img border='0' src='../human/reddot.gif'></A>",
#	"<A href='alignment.cgi?gid=$gid&class=$class_new&seq=aa' target='_new'>",
#	"<img width='12' border='0' src='../human/yellowdot.gif'></A></TD>";

       #print "<TD align='center'><FONT SIZE='-2' CLASS='sans-serif' FACE='arial, helvetica'>$acc</FONT></TD>",
	print "<TD align='center'><font size='-2' CLASS='sans-serif' FACE='arial, helvetica'><A href='pgene.cgi?id=$id&tax_id=$tax_id&organism=$org' target='_new'>$acc</font></TD>",
	"<TD><FONT SIZE='-2' CLASS='sans-serif' FACE='arial, helvetica'>$chr_num</FONT></TD>",
	"<TD><FONT SIZE='-2' CLASS='sans-serif' FACE='arial, helvetica'>$start to $stop</FONT></TD>",
        "<TD><FONT SIZE='-2' CLASS='sans-serif' FACE='arial, helvetica'>$p_acc</FOnT></TD>",
	"<TD><FONT SIZE='-2' CLASS='sans-serif' FACE='arial, helvetica'>$p_start to $p_stop</FONT></TD>",
	"<TD><FONT SIZE='-2' CLASS='sans-serif' FACE='arial, helvetica'>$e_value</FONT></TD>",
	"<TD><FONT SIZE='-2' CLASS='sans-serif' FACE='arial, helvetica'>$aa_identity</FONT></TD>",
	"<TD><FONT SIZE='-2' CLASS='sans-serif' FACE='arial, helvetica'>".($polya || 'NULL')."</FONT></TD>",
	"<TD><FONT SIZE='-2' CLASS='sans-serif' FACE='arial, helvetica'>".($disable || 'NULL')."</FONT></TD>";	
    }
    print "</table><FONT SIZE='-1' CLASS='sans-serif' FACE='arial, Helvetica'>",
    "<P><center>$length total annotations found.</center></P></FONT>",
    
    "</FONT></body></html>";
    
}

