#!/usr/bin/perl -w
use strict;
use DB::db;
use CLASSES::PFSet;
use CLASSES::Pseudogene;
use CLASSES::Fragment;
use CLASSES::Build;
use ALIGNMENT::default_align;
use QUERY::query_factory;

my $name = "Pipeline Union Set";

my $tax_id = 9606; 
my $dbh = DB::db::getDB(db=>"Pgenes_$tax_id");

my $build = CLASSES::Build->current_build_number($dbh);

my @P = sort cmp_pgene CLASSES::Pseudogene->get(dbh=>$dbh, where => "source = 'pseudogene.org'");
my @results = ();

my $i=0;
while ($i < @P) {
    my @coords1 = $P[$i]->get_coords();
    my $j = $i+1;
    $j++ while ($j < @P && $P[$i]->get_chromosome()->get_number() eq $P[$j]->get_chromosome->get_number() && ($P[$j]->get_coords())[0] <= $coords1[1]);

    # Select the first pseudogene.org pgene, if it exists
    my $selected = -1;
    for (my $k = $i; $k < $j && $selected < 0; $k++) {
	if ($P[$k]->get_source() eq 'pseudogene.org') {
	    $selected = $k;
	}
    }

    if ($selected < 0) {
	$selected = $i;
    }

    push(@results, $P[$selected]);

    my @coords3 = $P[$selected]->get_coords();
    $j++ while ($j < @P && $P[$i]->get_chromosome()->get_number() eq $P[$selected]->get_chromosome->get_number() && ($P[$j]->get_coords())[0] <= $coords3[1]);

    $i = $j;
}

my @S = CLASSES::PFSet->get(dbh=>$dbh, where=>"name = '$name'");
my $S = CLASSES::PFSet->new(dbh=>$dbh, name=>$name, notes=>"Canonical set of non-overlapping pseudogenes derived from the pseudogene.org pipeline",   reference=>"www.pseudogene.org", ref_address=>"http://www.pseudogene.org/", detailed_notes=>"Formed by taking the set of all pseudogenes identified by the pseudogene.org pipeline and iterativly removing overlaps until reaching a canonical set", acc=> @S > 0 ? $S[0]->get_acc() : undef, build => $build, pgene_list=>\@results);

$S->insert();

sub cmp_pgene {
    my ($s1, $f1) = $a->get_coords();
    my ($s2, $f2) = $b->get_coords();

    return $a->get_chromosome()->get_number() cmp $b->get_chromosome()->get_number()  ||  $s1 <=> $s2 || $f2 <=> $f1 || $a->get_acc() <=> $b->get_acc();
}
