#!/usr/bin/perl -w
use strict;
use CGI qw(:all);
use CLASSES::Pseudogene;
use DB::db;
use TaxID::tax_id;

print header();

eval {
    my $full_id = param("id") || "PGO.9606.7187"; #die("No acc specified.|");
    my $tax_id = param("tax_id");
    my $lsid = param("lsid") || param("transcript_id");
    my $id = param("internal_id");

    my $acc;
    if (!$id) {
	if ($full_id) {
	    $full_id =~ /^PGO\.*(\d+)\.(\d+)/ || die("Bad acc specified: $full_id.|");
	    $tax_id = $1;
	    $acc = $2;
	}
	else {
	    $acc = CLASSES::Pseudogene->parse_lsid($lsid);
	}
    }

    my $org_type = get_type($tax_id);
    my $org = get_org($tax_id);

    print "<html>";
    
    my $dbh;
    eval {
	$dbh = DB::db::getDB(db => "Pgenes_$tax_id");
    };
    if ($@) {
	die("Bad tax. id: $tax_id.|");
    }

    my $P = $id ? CLASSES::Pseudogene->get_by_id($dbh, $id) : CLASSES::Pseudogene->get_by_acc($dbh, $acc);
    die("No pgene exists with accession number $acc.|") unless $P;

    my $id = $P->get_id();
    my $lsid = $P->get_lsid();
    my $name = $P->pgene_name();
    my $Protein = $P->get_protein();
    my $p_acc = ref($Protein) && ref($Protein) eq "CLASSES::Protein" ? $Protein->get_acc() : "";
    my $p_name = ref($Protein) && ref($Protein) eq "CLASSES::Protein" ? $Protein->get_name() : "";
    my $chr_num = $P->get_chromosome()->get_number();
    my ($start, $stop) = $P->get_coords();
    my $strand = $P->get_strand;
    my $type = ($P->get_ptype() eq 'p') ? 'Processed' : 'Duplicated';
    my @A = $P->get_alignments();
    my ($p_start, $p_stop) = ($A[0]->get_protein_start(), $A[-1]->get_protein_finish());
    my $e_value = (@A==1) ? ($A[0]->get_e_value() || "NULL") : "NA";
    my $aa_identity = (@A == 1) ? ($A[0]->get_aa_ident() || "NULL") : "NA";
    my $polya = $P->get_polya();
    my $disable = $P->get_disable();
    
    my $aa = $P->get_aligned_translation();
    my $dna = $P->get_sequence();
    
    my $build_num = $P->get_chromosome()->get_build()->get_build_number();

#get the general information of the functional gene
    
    print "<A href=\"/cgi-bin/db-gen.cgi?org_type\"><P><CENTER><IMG src=\"/pgtop.jpg\"></P><P>Return to Top</P></A></CENTER>";

    print "<table width='1000'>";
    print "<td>";
    print "<FORM METHOD='POST' ACTION='dna_pr_display.cgi'>",
    "<INPUT type='hidden' name='acc' value='$acc'>",
    "<INPUT type='hidden' name='seq' value='$dna'>",
    "<INPUT type='submit' name='c_type' value='Nucleotide Sequence'>&nbsp;&nbsp;&nbsp;",
    "</FORM>";
    print "</td><td>";
    
    print "<FORM METHOD='POST' ACTION='dna_pr_display.cgi'>",
    "<INPUT type='hidden' name='acc' value='$acc'>",
    "<INPUT type='hidden' name='seq' value='$aa'>",
    "<INPUT type='submit' name='c_type' value='Amino Acid Sequence'>&nbsp;&nbsp;&nbsp;",
    "</FORM>";

    print "</td><tr><td></td>";

    print "<tr><td width='250'><B>Build:</td>";
    print "<td>$org $build_num</td></tr>";
    print "<tr><td width='250'><B>Pgene Acc:</td>";
    print "<td>$lsid</td></tr>";
    print "<tr><td width='150'><B>Pgene Name:</td>";
    print "<td>$name</td></tr>";
    print "<tr><td width='150'><B>Protein Acc:</td>";
    print "<td>$p_acc</td></tr>";
    print "<tr><td width='150'><B>Protein Name:</td>";
    print "<td>$p_name</td></tr>";
    print "<tr><td width='150'><B>Chromosome:</td>";
    print "<td>$chr_num</td></tr>";
    print "<tr><td width='150'><B>Chromosome Location:</td>";
    print "<td>$start-$stop</td></tr>";
    print "<tr><td width='150'><B>Query Location:</td>";
    print "<td>$p_start-$p_stop</td></tr>";
    print "<tr><td width='150'><B>E-value:</td>";
    print "<td>$e_value</td></tr>";
    print "<tr><td width='150'><B>Amino Acid Identity:</td>";
    print "<td>".(sprintf("%d", 100*($aa_identity+0.005))/100)."</td></tr>";
    print "</table>";

    print "<hr>";
    print "<b>Key:</b><br>";
    print "<br><b>Pgene Acc:</b> The pseudogene accession number (as assigned by the pseudogenes database)";
    print "<br><b>Pgene Name:</b> The pseudogene name (as assigned by the pseuogenes database)";
    print "<br><b>Protein Acc:</b> Accesion number of the generating protein (as assigned by the SwissProt or Ensabmle database)";
    print "<br><b>Chromosome:</b> The number of the chromosome containing the pseudogene";
    print "<br><b>Chromosome Location:</b> Start/stop coordinates on the chromosome (numbered from 1)";
    print "<br><b>Query Location:</b> Start/stop coordinates of the generating fragment of the protein (numbered from 1)";
    print "<br><b>E-value:</b> The e-value of the sequence match to the generating protein";
    print "<br><b>Amino Acid Identity:</b> The precentage of conserved amino acids in the protein/pseudogene alignment";
    print "</body></html>";
};
if ($@) {
    my $error = $@;
    $error =~ s/\|.*$//;

    print "<BR><BR>";
    print "<FONT SIZE='4' color='red' CLASS='sans-serif' FACE='arial,helvetica'>ERROR: $error</FONT><BR>";
}
    
print end_html();
    
