#!/usr/bin/perl

use DBI;
use DB::db;
use CLASSES::Pseudogene;
use CGI;
use TaxID::tax_id;
use strict;

my $q = new CGI;
#################################################
# Step 1: Get basic information for page and set up the database
my $tax_id = $q->param("tax_id") || 9606;
my $org = get_org($tax_id);

my $extended = $q->param("extended");  # Show (all) exended attributes if true
my $align = $q->param("align");        # Show alignment information if true

my $dbh = DB::db::getDB(db => "Pgenes_$tax_id");  # Set up the database handel

#################################################
# Step 2: Read in identifying information
# Three ways to identify pgene:
#    1) With the id (using parameter "internal_id")
#    2) With the acc (using parameter "acc" or "id")
#       Returns most current version.
#       Note: Need to use the "id" parameter to indeicate acc
#             to retain compatibility with UCSC link-back.
#    3) With the acc/version pair (using paramters
#       "acc" and "verson"

my $id = $q->param("internal_id") || 27600;   # $id should be defined iff $acc/$version are not
my $acc = $q->param("acc");  
   $acc = CLASSES::Pseudogene->parse_lsid($dbh, $acc) if $acc;
my $version = $q->param("version");

################################################
# Step 3: Pull pseudogene from database

my $P;       # Pseudogene object
if ($id) {
    $P = CLASSES::Pseudogene->get_pk($dbh, $id);
}
else {
    if ($version) {
	# Need the all_versions parameter in order to return a non-current version
	$P = CLASSES::Pseudogene->get(dbh=>$dbh, where=>"acc=$acc and version=$version", "all_versions"=>1);
    }
    else {
	$P = CLASSES::Pseudogene->get(dbh=>$dbh, $acc);
    }
}   


#####################################################
# Step 4: Get the relavant pseudogene information

# Get identifying information
my $id = $P->get_id();             # Pgene id.
my $acc = $P->get_lsid();          # Pgene acc.
my $name = $P->pgene_name();       # Pgene name.

# Get position information
my $chr_num = $P->get_chromosome()->get_number();
my ($start, $stop) = $P->get_coords();
my $strand = $P->get_strand;

# Get characteristic information
my $type = $P->get_ptype();
my $gc_content = sprintf("%0.2f", $P->get_GC_Pgene());

# Get Protein/Gene information
my $Protein = $P->get_protein();   # Protein name -- protein could be unknown
   $Protein = undef unless ref($Protein) eq "CLASSES::Protein";   # Make sure its a Protein object or undefined   
my $p_acc = $Protein ? $Protein->get_acc() : "";
my $p_name = $Protein ? $Protein->get_name() : "";
my $gene_id = $Protein ? $Protein->get_gene_id() : "";

# Get sequence information
my $aa = $P->get_aligned_translation();
my $dna = $P->get_sequence();

# Get alignment information (if necessary)
my @A = $P->get_alignments();
my ($p_start, $p_stop) = ($A[0]->get_protein_start(), $A[-1]->get_protein_finish());
my $e_value = (@A==1) ? ($A[0]->get_e_value() || "") : "";
my $aa_identity = (@A == 1) ? ($A[0]->get_aa_ident() || "") : "";
my $dna_identity = (@A == 1) ? ($A[0]->get_dna_ident() || "") : "";

my @protein_align_seq;  # Protein alignment string (by exon)
my @frag_align_seq;     # Fragment alignment string (by exon)
for (@A) {
    push(@protein_align_seq, $_->get_protein_align_seq());
    push(@frag_align_seq, $_->get_frag_align_seq());
}

# Get other information
my $web_link = $P->get_web_link();

# Start printing information
print $q->header();
print $q->start_html();
print $q->head($q->title("Pseudogene Display ($org: $acc)"));
print "<body>";
#get the general information of the functional gene

#print "<FONT SIZE='6pt' color='red'><B><p>$name</p></font>";
#print '<TABLE CLASS="table01" width=102%><TR><TD>';
print "<B>Pseudogene Information:</B>";
print "<table width='1000'>";
print "<tr><td width='250'>Pseudogene Accession Number:</td>",
      "<td>$acc</td></tr>";
print "<tr><td width='150'>Pseudogene Name:</td>",
      "<td>$name</td></tr>";
print "<tr><td width='150'>Pseudogene Location:</td>",
      "<td>chr".$chr_num.":$start-$stop</td></tr>";
print "<tr><td width='150'>Protein Accession Number:</td>",
      "<td>$p_acc</td></tr>";
print "<tr><td width='150'>Protein Name:</td>",
      "<td>$p_name</td></tr>";
print "<tr><td width='150'>Gene ID:</td>",
      "<td>$gene_id</td></tr>";
print "<tr><td width='150'>GC Content:</td>",
      "<td>$gc_content</td></tr>";

if ($web_link) {
    print "<tr><td width='150'>External Link:</td>",
          "<td><a href='$web_link'>Click here</td>";
}
print "</table>";
print "</tr></td>";
print "</table>";
print "<BR>";


if ($align) {
    print "<B>Sequence Information</B>";
    print "<table width='1000'>";
    print "<tr><td width='250'>Aligning tool: </td>",
          "<td>".$A[0]->get_tool()."</td>";
    print "<tr><td width='250'>Protein Coordinates: </td>",
          "<td>$p_start-$p_stop</td>";
    print "<tr><td width='250'>e-value: </td>",
          "<td>$e_value</td>";
    print "<tr><td width='250'>Amino acid identity: </td>",
          "<td>$aa_identity</td>";
    print "</table>";
    print "Alignment:<br>";
    print $q->br(join($q->b(" ... "), @protein_align_seq));
    print $q->br(join($q->b(" ... "),, @frag_align_seq));
    print "<br>";
}

print "<BR>";

if ($extended) {
    print "<B>Extended Attributes:</B>";
    my %H = $P->get_extension_attributes();
    if (keys %H == 0) {
	print "<br>None<br>";
    }
    else {
	print "<table width='1000'>";
	for (sort keys %H) {
	    my ($att, $value) = ($_, $H{$_});
	    print"<tr><td width = '250'>$att:</td>",
	    "<td>$value</td></tr>";
	}
    }
    print "</table><BR>";
}

if ($align) {
    print "<FORM METHOD='POST' ACTION='pgene.cgi'>",
          "<INPUT type = 'hidden' name = 'tax_id' value=$tax_id>",
          "<INPUT type = 'hidden' name = 'extended' value=$extended>",
          "<INPUT type = 'hidden' name = 'id' value=$id>",
          "<INPUT type = 'hidden' name = 'align' value=0>",
          "<INPUT type = 'submit' name = 'junk' value = 'Remove Alignment Information'>",
          "</FORM>";
}
else {
    print "<FORM METHOD='POST' ACTION='pgene.cgi'>",
          "<INPUT type = 'hidden' name = 'tax_id' value=$tax_id>",
          "<INPUT type = 'hidden' name = 'extended' value=$extended>",
          "<INPUT type = 'hidden' name = 'id' value=$id>",
          "<INPUT type = 'hidden' name = 'align' value=1>",
          "<INPUT type = 'submit' name = 'junk' value = 'Display Alignment Information'>",
          "</FORM>";
}

if ($extended) {
    print "<FORM METHOD='POST' ACTION='pgene.cgi'>",
          "<INPUT type = 'hidden' name = 'tax_id' value=$tax_id>",
          "<INPUT type = 'hidden' name = 'extended' value=0>",
          "<INPUT type = 'hidden' name = 'id' value=$id>",
          "<INPUT type = 'hidden' name = 'align' value=$align>",
          "<INPUT type = 'submit' name = 'junk' value = 'Remove Extended Attributes'>",
          "</FORM>";
}
else {
    print "<FORM METHOD='POST' ACTION='pgene.cgi'>",
          "<INPUT type = 'hidden' name = 'tax_id' value=$tax_id>",
          "<INPUT type = 'hidden' name = 'extended' value=1>",
          "<INPUT type = 'hidden' name = 'id' value=$id>",
          "<INPUT type = 'hidden' name = 'align' value=$align>",
          "<INPUT type = 'submit' name = 'junk' value = 'Display Extended Attributes'>",
          "</FORM>";
}

print "</table></center></body>";
print $q->end_html();

