#!/usr/bin/perl -w
use strict;

package functions::fun;
require Exporter;

our @ISA = qw(Exporter);
our @EXPORT = qw(string_gc pgene_gc fragment_gc check_polyA);
our $VERSION = 1.00;

sub string_gc {
    my $str = shift;
    $str =~ s/N//g;
    return undef if (!defined($str) or length($str)==0);

    my $str2 = $str;
    $str2 =~ s/[atAT]//g;
    return length($str2)/length($str);
}

sub pgene_gc {
    my $pgene = shift;
    (ref($pgene) eq "CLASSES::Pseudogene") or die("fun::Not a pgene object\n");

    return string_gc($pgene->get_sequence());
}

sub fragment_gc {
    my $fragment = shift;
    (ref($fragment) eq "CLASSES::Fragment") or die("fun::Not a pgene object\n");

    return string_gc($fragment->get_sequence());
}
		 
sub check_polyA {
    # $P = pseudogene object
    # $cutoff = minimum number of As in the polya tail
    # $window_size = size of a window in which to count the number of As
    # $block_size = size of block following the seqeunce for which we will search for a polyA tail
    my ($P, $cutoff, $window_size, $block_size) = @_;
    ($window_size < $block_size) or die("check_polyA: bad window/block size\n");

    # Get the mRNA sequence translation of the sequence of size $block_size following the sequence
    my $sequence;
    my @coords = $P->get_coordinates();
    my $C = $P->get_chromosome();
    if ($P->get_strand() eq '+') {
	$sequence = $C->get_sequence($coords[1]+1, &min($coords[1]+$block_size, $P->get_chromosome()->get_genome()->get_size()-1), '+');    
    }
    else {
	$sequence = $C->get_sequence(&max($coords[0]-$block_size, 1), $coords[0]-1, '-');       
    }
    $sequence = TOOLS::sequence_functions::get_mRNA($sequence);
    $sequence =~ tr/a/A/;   #Makes life easier below

#    print $sequence."\n";

    # count the number of As in the first substring of size $window_size
    my @arr = substr($sequence, 0, $window_size) =~ /A/g;
    my $num = @arr;

    # scan through the sequence to count the number of As in every substring of size $window_size
    # record the position of the (first) substring with the most number of As in $max_position
    my $max_num = $num;
    my $max_position = 0;
    for (my $i=1; $i < length($sequence)-$window_size; $i++) {
	$num = $num - (substr($sequence, $i-1, 1) eq "A" ? 1 : 0) + (substr($sequence, $i+$window_size-1, 1) eq "A" ? 1 : 0);
	($max_num, $max_position) = ($num, $i) if $num > $max_num;
    }

    # if the most number of As found was less than the cutoff, quit
    (return (wantarray ? (0, undef, $max_position, $max_num) : 0)) if ($max_num < $cutoff);

    # find the position of the polyadenilation signal
    my $str = substr($sequence, 0, $max_position);
    my ($class, $signal_dist, $signal_pos);
    if ($str =~ /^(\w*)(AAUAAA|AUUAAA|CAUAAA|AAUAUA)/i) {
	$signal_pos = length($1);
	$signal_dist = length($str) - $signal_pos - length($2);
	if ($signal_dist < 50) {
	    $class = 1;
	}
	elsif ($signal_dist <=100) {
	    $class = 2;
	}
	else {
	    $class = 3;
	}
    }
    else {
	$signal_pos = undef;
	$class = 3;
    }

    return wantarray ? ($class, $signal_pos, $max_position, $max_num) : $class;
}

sub min {
    return $_[0] < $_[1] ? $_[0] : $_[1];
}

sub max {
    return $_[0] <= $_[1] ? $_[1] : $_[0];
}
