#!/usr/bin/perl -w
use strict;

use CLASSES::Pseudogene;
use DB::db;

open(OFILE, ">analysis_results.txt");

my $dbh = DB::db::getDB({"db" => "Pgenes_9606"});

my @all = CLASSES::Pseudogene->get($dbh);

my @polya_arr;
for (my $i=0; $i <= 3; $i++) {
    my @arr = CLASSES::Pseudogene->search_on_attributes($dbh, "polya = $i");
    push(@polya_arr, \@arr);
}

my %disable;
my $total = 0;

foreach (("0", "d", "h")) {
    my @arr = CLASSES::Pseudogene->search_on_attributes($dbh, "disable = $_");
    $disable{$_} = \@arr;
}

my $avg_gc = 0;
my $max_gc = $all[0];
my $min_gc = $all[0];
my $undefined_gc = 0;
foreach (@all) {
    my $v = $_->get_GC_Pgene();
    if (defined $v) {
	$avg_gc += $v;
	if ($max_gc->get_GC_Pgene() < $v) {
	    $max_gc = $_;
	}
	elsif ($min_gc->get_GC_Pgene() > $v) {
	    $min_gc = $_;
	}
    }
    else {
	$undefined_gc++;
    }
}
$avg_gc = $avg_gc / (@all - $undefined_gc);
    
my $var_gc = 0;
foreach (@all) {
    next unless defined $_->get_GC_Pgene();
    $var_gc += abs($_->get_GC_Pgene() - $avg_gc);
}
my $sd_gc = sqrt($var_gc) / (@all - $undefined_gc);


my $avg_iso_gc = 0;
my $max_iso_gc = $all[0];
my $min_iso_gc = $all[0];
my $undefined_iso = 0;
foreach (@all) {
    my $v = $_->get_GC_Isochore();
    if (defined $v) {
	$avg_iso_gc += $v;
	if ($max_iso_gc->get_GC_Isochore() < $v) {
	    $max_iso_gc = $_;
	}
	elsif ($min_iso_gc->get_GC_Isochore() > $v) {
	    $min_iso_gc = $_;
	}
    }
    else {
	$undefined_iso++;
    }
}
$avg_iso_gc = $avg_iso_gc / (@all - $undefined_iso);
    
my $var_iso_gc = 0;
foreach (@all) {
    next unless defined $_->get_GC_Isochore();
    $var_iso_gc += abs($_->get_GC_Isochore() - $avg_iso_gc);
}
my $sd_iso_gc = sqrt($var_iso_gc) / (@all - $undefined_iso);



#-----------------------------------------
print OFILE "Num. pgenes: ".@all."\n\n";

print OFILE "poly:\n";
for (my $i=0; $i <= 3; $i++) {
    print OFILE "$i: ".@{$polya_arr[$i]}." (".$polya_arr[$i]->[0]->get_id().")\n";
}
print OFILE "\n";

print OFILE "disable:\n";
while (my ($k, $v) = each %disable) {
    print OFILE "$k: ".@$v."\n";
}
print OFILE "\n";

print OFILE "GC Content:\n";
print OFILE "Average: $avg_gc\n";
print OFILE "S.D.: $sd_gc ($var_gc)\n";
print OFILE "Max: ".$max_gc->get_GC_Pgene()." (".$max_gc->get_id().")\n";
print OFILE $max_gc->get_sequence()."\n";
print OFILE "Min: ".$min_gc->get_GC_Pgene()." (".$min_gc->get_id().")\n";
print OFILE $min_gc->get_sequence()."\n";

print OFILE "ISO_GC Isochore Content:\n";
print OFILE "Average: $avg_iso_gc\n";
print OFILE "S.D.: $sd_iso_gc ($var_iso_gc)\n";
print OFILE "Max: ".$max_iso_gc->get_GC_Isochore()." (".$max_iso_gc->get_id().")\n";
print OFILE $max_iso_gc->get_sequence()."\n";
print OFILE "Min: ".$min_iso_gc->get_GC_Isochore()." (".$min_iso_gc->get_id().")\n";
print OFILE $min_iso_gc->get_sequence()."\n";

print join(" ", map {$_->get_id()} @{$polya_arr[1]})."\n";
