#!/usr/bin/perl -w
use strict;

my $file = shift @ARGV;
$file =~ /^(.+)\.cl/;
my $aug_file = "$1-aug.cl";

my $line;

my $dir;
my $name;
my %H;

my @arr;

open(IFILE, $file);
chomp($line = <IFILE>);
@arr = split(/\t+/, $line);
$dir = $arr[1] eq "." ? undef : $arr[1];

chomp($line = <IFILE>);
@arr = split(/\t+/, $line);
$name = $arr[1];

my @class_attributes;
foreach (<IFILE>) {
    next unless /^att/;
    chomp;
    @arr = split(/\t+/);
    $H{$arr[1]} = $arr[2];
    push(@class_attributes, $arr[1]);
}
close(IFILE);


my $ofile = defined($dir) ? "$dir/$name\.pm" : $name.".pm";
my $pack  = defined($dir) ? "$dir\:\:$name" : $name;

my $att_str = "qw(@class_attributes)";
my $insert_str = "insert into $name(".join(", ", @class_attributes).") values(".("?, " x ((scalar keys %H)-1))."?)";
my $select_str = "select ".join(",", @class_attributes)." from $name";
my $update_str = "update $name set ".join("=?,", @class_attributes)."=? where id=?";

open(OFILE, ">$ofile");

#-------------------------------------------------------------
print OFILE <<START_OF_HEADING;
package $pack;
our \$VERSION = '1.0';
use strict;
use TOOLS\:\:get_date_string;
use CLASSES\:\:ClassFun;
use DB\:\:Cache;

my \@attributes = $att_str;
my \$insert_query = "$insert_str";
my \$select_query = "$select_str";
my \$update_query = "$update_str";

sub $name\_isAtt {
    (\@_ == 1) && !ref(\$_[0]) or die("$pack\:\:$name\_isAtt: Unknown paramter\\n");
    foreach (\@attributes) {
	return 1 if \$_ eq \$_[0];
    }
    return 0;
}
START_OF_HEADING

#-----------------------------------------------------------
print OFILE <<IMPORTANT_METHODS;

\#----- Static Methods

\#######################
\# Name: get_attribute_list
\# Purpose: Static/dynamic method that returns a list of all the class attributes.
\# Parameters: None
\# Output: List of attributes
sub get_attribute_list {
    return \@attributes;
}

\#######################
\# Name: create
\# Purpose: Static/dynamic method that inserts the values of one or more objects into new rows in the database
\# Parameters: Database handle
\#             List of objects (for static version)
\# Output: Number of lines affected
\# Side effects: Modifies the objects primary keys (if necessary)
sub create { 
    my \$proto = shift;
    my \$class = ref(\$proto) || \$proto;
    
    (ref(\$proto) && \$proto->is_in_db()) and die("$pack\:\:create: Cannot call dynamically on an object already in the database\\n");
    my \@obj_array = ref(\$proto) ? \$proto : \@_;
    my \$date = get_date_string::get_date_string();
    foreach (\@obj_array) {
	\$_->set_entry_date(\$date);
    }

    my \$value = CLASSES::ClassFun::insert_class(\$insert_query, \@obj_array);
    foreach (\@obj_array) {
	\$_->mark_current();
	\$_->mark_in_db();
	DB::Cache->set(\$_->get_dbh(), "$name", \$_->get_id(), \$_);
    }
    return \$value;
}

\#######################
\# Name: update
\# Purpose: Static/dynamic method that updates the values of one or more objects in the database
\# Parameters: Database handle
\#             List of objects (for static version)
\# Output: Number of lines affected
\# Side effects: Modifies the objects primary keys (if necessary)
sub update { 
    my \$proto = shift;
    my \$class = ref(\$proto) || \$proto;
    
    my \@obj_array = ref(\$proto) ? (\$proto) : \@_;
    foreach (\@obj_array) {
	\$_->mark_current();
	DB::Cache->set(\$_->get_dbh(), "$name", \$_->get_id(), \$_);
    }
    return update_class(\$update_query, \@obj_array);
}

\#######################
\# Name: get
\# Purpose: Static method to extract rows from a database
\# Paramters: Dataebase handle (static version only)
\#            where clause (static version only)
\# Output: A list of new objects conforming to the where statement (static version)
\#         None (dynamic version)
\# Side effects: In the dynamic version, the object is updated.  (Nothing is done
\#               if the select find 0 results or 2 or more results.)
sub get {
    my \$proto = shift;
    ref(\$proto) and die("$pack\:\:get: For static use only");
    my \$dbh = shift or die("$pack\:\:get: Unkown parameters\\n");
    (ref(\$dbh) eq "DBI\:\:db") or die("$pack\:\:get: Bad database handle\\n");
    
    my \$where_arr_ref = shift;
    my \$query = \$select_query.(\$where_arr_ref ? " where ".CLASSES::Clause::make_where(\$where_arr_ref) : "");
    
    my \@objs;
    foreach (CLASSES::ClassFun::get_class(\$dbh, \$query, \$where_arr_ref)) {
	my \$obj = DB::Cache->get(\$dbh, "$name", \$_->{"id"});
	unless (\$obj) {
	    \$obj = CLASSES::$name->new(\$dbh, \$_);
	    \$obj->mark_current();
	    \$obj->mark_in_db();
	    DB::Cache->set(\$dbh, "$name", \$obj->get_id(), \$obj);
	}
	push(\@objs, \$obj);
    }
    
    return wantarray ? \@objs : \$objs[0];
}

\#######################
\# Name: get_pk
\# Purpose: Static method to get information from tables based on the primary key
\# Paramters: Either: A single value
\#                    A hash reference containing all primary keys and values
\# Output: The object
sub get_pk {
    my \$proto = shift or die("$pack\:\:get_pk: Unknown parameters (1)\\n");
    ref(\$proto) and die("$pack\:\:get_pk: Can only be used as a static method\\n");
    my \$dbh = shift or die("$pack\:\:get_pk: Unkown parameters (2)\\n");
    my \$value = shift or die("$pack\:\:get_pk: Unkown parameters (3)\\n");

    return DB::Cache->get(\$dbh, "$name", \$value) || CLASSES\:\:$name->get(\$dbh, [{"id" => \$value}]);
}
	    
	

    

\#######################
\# Name: remove
\# Purpose: Static/dynamic method to remove rows from the table
\# Parameters: Database handle
\#             Where statement or 'all' (static version only)
\# Output: Objests for each line affected
sub remove {
    my \$proto = shift;

    if (ref(\$proto)) {          #Dynamic version
	(\@_ == 0) or die("$pack\:\:remove: Unknown parameters for dynamic form\\n");
	\$proto->is_in_db() or die("$pack\:\:remove: Cannot call dynamically on an object not in the database\\n");

	my \@where = ();
	foreach (CLASSES::$name->get_attribute_list()) {
	    if (\$proto->get_attribute(\$_)) {
		push \@where, {\$_, \$proto->get_attribute(\$_)};
		push \@where, "and";
	    }
	}
	pop(\@where);

	my \$query = "delete from $name where ".CLASSES::Clause::make_where(\\\@where);
	return CLASSES::ClassFun::remove_class(\$proto->get_dbh(), \$query, \\\@where);
    }
    else {                       #Static version
	my \$dbh = shift or die("$pack\:\:remove: Unkown parameters\\n");
	(ref(\$dbh) eq "DBI\:\:db") or die("$pack\:\:remove: Bad database handle\\n");
	my \$where_arr_ref = shift;
	(\$where_arr_ref && (\$where_arr_ref eq 'all' || ref(\$where_arr_ref) eq 'ARRAY')) or die("$pack\:\:remove: Most provide a (legal) where clause or string \\"all\\" to prevent accident deletion of whole table\\n");

	\$where_arr_ref = undef if \$where_arr_ref eq 'all';
	my \@objs = CLASSES\:\:$name->get(\$dbh, \$where_arr_ref);
	foreach (\@objs) {
	    \$_->unmark_current();
	    \$_->unmark_in_db();
	}

	my \$query = "delete from $name".(\$where_arr_ref ? " where ".CLASSES::Clause::make_where(\$where_arr_ref) : "");
	CLASSES::ClassFun::remove_class(\$dbh, \$query, \$where_arr_ref);

	return \@objs;
    }
}
	

\#------------------- Object methods

\#######################
\# Name: new
\# Purpose: Constructor
\# Input: Hash containing attribute/value pairs
sub new {
    my \$proto = shift;
    my \$class = \$proto;
    my \$dbh = shift or die("$pack\:\:new: Unknown parameters");
    my \$self = {};
    my \$params = shift or die("$pack\:\:new: Unknown prameters");

    foreach (\@attributes) {
	\$self->{"_\$_"} = exists \$params->{\$_} ? \$params->{\$_} : undef;
    }
    \$self->{"_entry_date"} = get_date_string\:\:get_date_string() unless defined(\$self->{"_entry_date"});

    foreach (keys \%\$params) {
	exists \$self->{"_\$_"} or die("$pack\:\:new: Bad hash field: \$_\\n");
    }

    \$self->{"_dbh"} = \$dbh;
    \$self->{"_current"} = 0;  # True if identical to the copy in the db
    \$self->{"_in_db"} = 0;    # True if there is a version in the db
	
    bless \$self, \$class;
}

\#######################
\# Name: Get_attribute:
\# Purpose: Get the value of a given attribute
\# Paramters: Attribute name
\# Output: Attribute value
sub get_attribute {
    my \$self = shift;
    my \$att = shift;

    $name\_isAtt(\$att) or die("$pack\:\:attribute: Bad attribute: \$att\\n");
    my \$method = "get\_\$att";
    return \$self->\$method();
}

\#######################
\# Name: set_attribute
\# Purpose: Set attributes to a given value
\# Paramters: Attribute name
\#            value
\# Output: None
sub set_attribute {
    my \$self = shift;
    my \$att = shift;
    my \$value = shift;

    $name\_isAtt(\$att) or die("$pack\:\:set_attribute: Bad attribute: \$att\\n");
    my \$method = "set\_\$att";
    \$self->\$method(\$value);
}

\#######################
\# Name: get_dbh
\# Purpose: get the class database handle
\# Paramters: None
\# Output: None
sub get_dbh {
    return \$_[0]->{"_dbh"};
}

\#######################
\# Name: mark/unmark/flip_current, get_current
\# Purpose: toogle the current marker, check the current marker
\# Paramters: None
\# Output: None, current value
sub mark_current {
    \$_[0]->{"_current"} = 1;
}

sub unmark_current {
    \$_[0]->{"_current"} = 0;
}

sub toggle_current {
    \$_[0]->{"_current"} = 1 - \$_[0]->{"_current"};
}

sub is_current {
    return \$_[0]->{"_current"};
}

\#######################
\# Name: mark/unmark/flip_in_db, is_in_db
\# Purpose: toogle, check if in database
\# Paramters: None
\# Output: None, current value
sub mark_in_db {
    \$_[0]->{"_in_db"} = 1;
}

sub unmark_in_db {
    \$_[0]->{"_in_db"} = 0;
}

sub toggle_in_db {
    \$_[0]->{"_in_db"} = 1 - \$_[0]->{"_current"};
}

sub is_in_db {
    return \$_[0]->{"_in_db"};
}


\#######################
\# Name: str
\# Purpose: Return a printable string discribing the object contents
\# Paramters: None
\# Output: The string
sub str {
    my \$self = shift;
    my \@values = ();

    foreach (\$self->get_attribute_list()) {
	my \$value = \$self->get_attribute(\$_) || ".";
	push(\@values, \$value);
    }
    return join("\\t", \@values);
}

IMPORTANT_METHODS
#-----------------------------------------------------------
foreach (keys %H) {

print OFILE <<PRINT_SET_GET_METHODS;
\#######################    
sub get_$_ {
    my \$self =  \$_[0];
    return \$self->{"_$_"};
}

sub set_$_ {
    my \$self =  \$_[0];
    \$self->unmark_current();
    \$self->{"_$_"} = \$_[1];
}

sub $_\_type {
    return "$H{$_}";
}

PRINT_SET_GET_METHODS
}
#--- Append other stuff on
if (-e $aug_file) {
    print OFILE "\n\n\#\#\#\#\#\#\# Class specific methods\n";
    open(IFILE, $aug_file);
    while (<IFILE>) {
	print OFILE $_;
    }
    close(IFILE);
}

print OFILE <<THE_ONE
1;
THE_ONE


