#!/usr/bin/perl

use DBI;
use DB::db;
use CLASSES::Pseudogene;
use CGI;
use TaxID::tax_id;
use strict;

my $q = new CGI;
#################################################
# Step 1: Get basic information for page and set up the database
my $tax_id = $q->param("tax_id");

my $extended = $q->param("extended");  # Show (all) exended attributes if true
my $align = $q->param("align");        # Show alignment information if true
my $lsid = $q->param("lisd");           # Tax id may be imbeded

my $acc;
if (my $temp = "PGO.9606.10") { #$q->param("id")) {
    $temp =~ /^PGO\.(\d+)\.(\d+)$/;
    ($tax_id, $acc) = ($1, $2);
}
elsif ($lsid && $lsid =~ /urn\:lsid\:pseudogene.org\:(\d+)\.Pseudogene\:(\d+)/) {
    $tax_id = $1;
    $acc = $2;
}

my $dbh = DB::db::getDB(db => "Pgenes_$tax_id");  # Set up the database handel
my $org = get_org($tax_id);

#################################################
# Step 2: Read in identifying information
# Three ways to identify pgene:
#    1) With the id (using parameter "internal_id")
#    2) With the acc (using parameter "acc" or "id")
#       Returns most current version.
#       Note: Need to use the "id" parameter to indeicate acc
#             to retain compatibility with UCSC link-back.
#    3) With the acc/version pair (using paramters
#       "acc" and "verson"

my $id = $q->param("internal_id");   # $id should be defined iff $acc/$version are not
$acc = $q->param("acc") unless $acc;
$acc = CLASSES::Pseudogene->parse_lsid($dbh, $acc) if $acc;

my $version = $q->param("version");

################################################
# Step 3: Pull pseudogene from database

my $P;       # Pseudogene object
if ($id) {
    $P = CLASSES::Pseudogene->get_pk($dbh, $id);
}
else {
    if ($version) {
	# Need the all_versions parameter in order to return a non-current version
	$P = CLASSES::Pseudogene->get(dbh=>$dbh, where=>"acc=$acc and version=$version", "all_versions"=>1);
    }
    else {
	$P = CLASSES::Pseudogene->get_by_acc($dbh, $acc);
    }
}   


#####################################################
# Step 4: Get the relavant pseudogene information

# Get identifying information
my $id = $P->get_id();             # Pgene id.
my $acc = $P->get_lsid();          # Pgene acc.
my $name = $P->pgene_name();       # Pgene name.

# Get position information
my $chr_num = $P->get_chromosome()->get_number();
my ($start, $stop) = $P->get_coords();
my $strand = $P->get_strand;

# Get characteristic information
my $type = $P->get_ptype();
my $gc_content = sprintf("%0.2f", $P->get_GC_Pgene());

# Get Protein/Gene information
my $Protein = $P->get_protein();   # Protein name -- protein could be unknown
   $Protein = undef unless ref($Protein) eq "CLASSES::Protein";   # Make sure its a Protein object or undefined   
my $p_acc = $Protein ? $Protein->get_acc() : "";
my $p_name = $Protein ? $Protein->get_name() : "";
my $gene_id = $Protein ? $Protein->get_gene_id() : "";

# Get sequence information
my $aa = $P->get_aligned_translation();
my $dna = $P->get_sequence();

# Get alignment information (if necessary)
my @A = $P->get_alignments();
my ($p_start, $p_stop) = ($A[0]->get_protein_start(), $A[-1]->get_protein_finish());
my $e_value = (@A==1) ? ($A[0]->get_e_value() || "") : "";
my $aa_identity = (@A == 1) ? ($A[0]->get_aa_ident() || "") : "";
my $dna_identity = (@A == 1) ? ($A[0]->get_dna_ident() || "") : "";

my @protein_align_seq;  # Protein alignment string (by exon)
my @frag_align_seq;     # Fragment alignment string (by exon)
for (@A) {
    push(@protein_align_seq, $_->get_protein_align_seq());
    push(@frag_align_seq, $_->get_frag_align_seq());
}

# Get other information
my $web_link = $P->get_web_link();

# Start printing information
print $q->header();
print $q->start_html();
print $q->head($q->title("Pseudogene Display ($org: $acc)"));
print "<body>";
#get the general information of the functional gene

#print "<FONT SIZE='6pt' color='red'><B><p>$name</p></font>";
#print '<TABLE CLASS="table01" width=102%><TR><TD>';
print "<B>Pseudogene Information:</B>";
print "<table width='1000'>";
print "<tr><td width='250'>Pseudogene Accession Number:</td>",
      "<td>$acc</td></tr>";
print "<tr><td width='150'>Pseudogene Name:</td>",
      "<td>$name</td></tr>";
print "<tr><td width='150'>Pseudogene Location:</td>",
      "<td>chr".$chr_num.":$start-$stop</td></tr>";
print "<tr><td width='150'>Protein Accession Number:</td>",
      "<td>$p_acc</td></tr>";
print "<tr><td width='150'>Protein Name:</td>",
      "<td>$p_name</td></tr>";
print "<tr><td width='150'>Gene ID:</td>",
      "<td>$gene_id</td></tr>";
print "<tr><td width='150'>GC Content:</td>",
      "<td>$gc_content</td></tr>";

if ($web_link) {
    print "<tr><td width='150'>External Link:</td>",
          "<td><a href='$web_link'>Click here</td>";
}

print "</table>";
print "</tr></td>";
print "</table>";
print "<BR>";


if ($align) {
    print "<B>Sequence Information</B>";
    print "<table width='1000'>";
    print "<tr><td width='250'>Aligning tool: </td>",
          "<td>".$A[0]->get_tool()."</td>";
    print "<tr><td width='250'>Protein Coordinates: </td>",
          "<td>$p_start-$p_stop</td>";
    print "<tr><td width='250'>e-value: </td>",
          "<td>$e_value</td>";
    print "<tr><td width='250'>Amino acid identity: </td>",
          "<td>$aa_identity</td>";
    print "</table>";
    print "Alignment:<br>";

  for(my $j = 0; $j < @protein_align_seq; $j++  ){
	my $protein_frag = $protein_align_seq[$j];
	my $pseudo_frag = $frag_align_seq[$j];
 	my @chars1 = split //, $protein_frag;
	my @chars2 = split //, $pseudo_frag;
    my $row1 = "P$j     "; my $row2 = "F$j  ";	
    for(my $i = 0; $i < @chars1;$i++){
	my $char1 = $chars1[$i]; my $char2 = $chars2[$i];
	if ($char1 ne $char2){
		$row1 .= "<font face='Courier'>$char1</font>"; $row2 .= "<font face='Courier'>$char2</font>";
	}else{
		$row1 .= "<font face='Courier' color=\"red\">$char1</font>";	
		$row2 .= "<font face='Courier' color=\"red\">$char2</font>";	
	}
	if (($i+1)%10 == 0){
		$row1 .= " "; $row2 .= " ";	
	} 	
	if (($i+1)%50 == 0){
		$row1 .= "\n"; $row2 .= "\n";
		print $q->br($row1);
		print $q->br($row2);
		print $q->br("\n");
    		$row1 = "P$j     ";  $row2 = "F$j  ";	
	}
    }
	print $q->br($row1."\n");
	print $q->br($row2."\n");
	print $q->br("\n");	
  }
    print "<br>";
}

print "<BR>";

if ($extended) {
    print "<B>Extended Attributes:</B>";
    my %H = $P->get_extension_attributes();
    if (keys %H == 0) {
	print "<br>None<br>";
    }
    else {
	print "<table width='1000'>";
	for (sort keys %H) {
	    my ($att, $value) = ($_, $H{$_});
	    print"<tr><td width = '250'>$att:</td>",
	    "<td>$value</td></tr>";
	}
    }
    print "</table><BR>";
}

if ($align) {
    print "<FORM METHOD='POST' ACTION='pgene.cgi'>",
          "<INPUT type = 'hidden' name = 'tax_id' value=$tax_id>",
          "<INPUT type = 'hidden' name = 'extended' value=$extended>",
          "<INPUT type = 'hidden' name = 'id' value=$id>",
          "<INPUT type = 'hidden' name = 'align' value=0>",
          "<INPUT type = 'submit' name = 'junk' value = 'Remove Alignment Information'>",
          "</FORM>";
}
else {
    print "<FORM METHOD='POST' ACTION='pgene.cgi'>",
          "<INPUT type = 'hidden' name = 'tax_id' value=$tax_id>",
          "<INPUT type = 'hidden' name = 'extended' value=$extended>",
          "<INPUT type = 'hidden' name = 'id' value=$id>",
          "<INPUT type = 'hidden' name = 'align' value=1>",
          "<INPUT type = 'submit' name = 'junk' value = 'Display Alignment Information'>",
          "</FORM>";
}

if ($extended) {
    print "<FORM METHOD='POST' ACTION='pgene.cgi'>",
          "<INPUT type = 'hidden' name = 'tax_id' value=$tax_id>",
          "<INPUT type = 'hidden' name = 'extended' value=0>",
          "<INPUT type = 'hidden' name = 'id' value=$id>",
          "<INPUT type = 'hidden' name = 'align' value=$align>",
          "<INPUT type = 'submit' name = 'junk' value = 'Remove Extended Attributes'>",
          "</FORM>";
}
else {
    print "<FORM METHOD='POST' ACTION='pgene.cgi'>",
          "<INPUT type = 'hidden' name = 'tax_id' value=$tax_id>",
          "<INPUT type = 'hidden' name = 'extended' value=1>",
          "<INPUT type = 'hidden' name = 'id' value=$id>",
          "<INPUT type = 'hidden' name = 'align' value=$align>",
          "<INPUT type = 'submit' name = 'junk' value = 'Display Extended Attributes'>",
          "</FORM>";
}

print "</table></center>";

   print "<hr>";
    print "<b>Key:</b><br>";
    print "<br><b>Pseudogene Accession Number:</b> The pseudogene accession number (as assigned by the pseudogenes database)";
    print "<br><b>Pseudogene Name:</b> The pseudogene name (as assigned by the pseuogenes database)";
    print "<br><b>Protein Accession Number:</b> Accesion number of the generating protein (as assigned by the SwissProt or Ensabmle database)";
    print "<br><b>Pseudogene Location:</b> The chromosome the pseudogene locates and its start/stop coordinates on the chromosome";
    print "<br><b>Protein Coordinates:</b> Start/stop coordinates of the generating fragment of the protein (numbered from 1)";
    print "<br><b>E-value:</b> The e-value of the sequence match to the generating protein";
    print "<br><b>Amino Acid Identity:</b> The precentage of conserved amino acids in the protein/pseudogene alignment";


print "</body>";
print $q->end_html();

