#!/usr/bin/perl -w

use strict;
use CLASSES::Build;
use CLASSES::Chromosome;
use CLASSES::Chromosome_Segment;
use TOOLS::genome;
use DB::db;

#Inputs: build nunber, tax id, directory

#@ARGV > 0 or die("Need command line arguments\n");

#################### (parameters)
my $build_number = shift @ARGV;
my $tax_id = shift @ARGV;
my $directory = shift @ARGV;
my $reset_tables = shift @ARGV || undef;

$directory = $directory."\/" unless substr($directory,length($directory)-1,1) eq "\/";

die("Bad reset tables value\n") unless !$reset_tables || ($reset_tables eq 'reset');
####################
if ($reset_tables) {
   system("./create_tables.pl $tax_id class-descriptions/Build.cl");
   system("./create_tables.pl $tax_id class-descriptions/Chromosome.cl");
   system("./create_tables.pl $tax_id class-descriptions/Chromosome_Segment.cl");
}
main(@ARGV);

######### MAIN
sub main {
    my $dbh = DB::db::getDB("db" => "Pgenes_$tax_id");

    my $build = create_build($dbh, $build_number, $directory);
    $build->insert();

    my @chr_arr;

    foreach (<$directory/chr*.fa>) {
	next if /random/;
	$_ =~ /(chr\w*.fa)/ || die("Bad file: $_\n");
	my $c = &create_chromosome($dbh, $build->get_id(), $1);
	$c->insert();
	my $chr_size = $c->get_size();
	my $frag_length = $c->get_genome()->get_seg_length();

	my @seg_arr;
	for (my ($i,$j)=(1,1); $i <= $chr_size; $i += $frag_length, $j++) {
	    push(@seg_arr, create_chromosome_segment($dbh, $c->get_id(), $j, $i, &min($i+$frag_length-1, $chr_size)));
	}
	CLASSES::Chromosome_Segment->insert(@seg_arr);	
    }

}

####################### Helpers
sub create_build {
    my $dbh = shift;
    my $build_number = shift;
    my $directory = shift;

    my $b = CLASSES::Build->new("dbh"=>$dbh, "build_number" => $build_number, "directory" => $directory, "tax_id" => $tax_id);

    return $b;
}

sub create_chromosome {
    my $dbh = shift || die("add_build::create_chomosome: Bad parameter: \$dbh missing");
    my $build_id = shift || die("add_build::create_chomosome: Bad parameter (build_id missing)\n");
    my $file_name = shift || die("add_build::create_chomosome: Bad parameter (file_name missing)\n");

    $file_name =~ /chr(\w+)\.fa$/ || die("add_build::create_chomosome: Bad file name: $_\n");
    my $number = "$1";

    my $c = CLASSES::Chromosome->new("dbh"=>$dbh, "build_id" => $build_id, "number" => $number, "file_name" => $file_name);

    return $c;
}

sub create_chromosome_segment {
    my $dbh = shift;
    my $chr_id = shift;
    my $position = shift;
    my $start = shift;
    my $finish = shift;

    my $s = CLASSES::Chromosome_Segment->new("dbh"=>$dbh, "chr_id" => $chr_id, "position" => $position, "start" => $start, "finish" => $finish);
    return $s;
}

sub min {
    return $_[0] <= $_[1] ? $_[0] : $_[1];
}
