#!/usr/bin/perl -w
use strict;

use DB::db;
use CLASSES::Build;
use CLASSES::Chromosome;
use CLASSES::Chromosome_Segment;
use CLASSES::Fragment;
use CLASSES::Pseudogene;
use CLASSES::Sequence_Alignment;
use CLASSES::Protein;
use CLASSES::PFSet;
use ALIGNMENT::default_align;
use TOOLS::assert;
use functions::fun;

######################################
# Add fragement data, pseudogene data
# and sequence_algingment data into 
# the appropriate tables;

######################### (parameters)
my $build_number = shift || 34;
my $tax_id = shift || 9606;
my $gff_file = shift || '/data/home/karro/mysql/data/human/build34/pgenes2/plus/chr1.sptrembl.all.gff';
my $fa_file = shift ||  '/data/home/karro/mysql/data/human/build34/pgenes2/plus/chr1.sptrembl.all.fa';
my $reset_tables = shift;

die("Bad reset tables value\n") unless !$reset_tables || ($reset_tables eq 'reset');

######################### (remove)
if ($reset_tables) {
   system("./create_tables.pl $tax_id class-descriptions/Fragment.cl");
   system("./create_tables.pl $tax_id class-descriptions/fragment_version.tbl");
   system("./create_tables.pl $tax_id class-descriptions/Pseudogene.cl");
   system("./create_tables.pl $tax_id class-descriptions/pseudogene_version.tbl");
   system("./create_tables.pl $tax_id class-descriptions/Sequence_Alignment.cl");
   system("./create_tables.pl $tax_id class-descriptions/fragment_to_pgene.tbl");
   system("./create_tables.pl $tax_id class-descriptions/pfset_to_pgene.tbl");
   system("./create_tables.pl $tax_id class-descriptions/pfset_to_fragment.tbl");
   system("./create_tables.pl $tax_id class-descriptions/pf_set_version.tbl");
   system("./create_tables.pl $tax_id class-descriptions/PFSet.cl");
   system("./create_tables.pl $tax_id class-descriptions/PFSet.cl");
   system("./create_tables.pl $tax_id class-descriptions/pseudogene_extension.tbl");
}

my $dbh = DB::db::getDB("db" => "Pgenes_$tax_id");    
my $build = CLASSES::Build->get_by_number($dbh, $build_number);


open(GFF_FILE, $gff_file) || die("GFF File not found: $gff_file\n") || die("Can't find GFF file: $_\n");
open(FA_FILE, $fa_file) || die("FA File not found: $fa_file\n") || die("Can't find FA file: $_\n");

my %gff_heading;
my $gff_head = <GFF_FILE>;
chomp($gff_head);
$gff_head =~ s/#//g; 
my $c = 0;
for (split(/\t/, $gff_head)) {
    $gff_heading{$_} = $c++;
}

my %unused = map {$_->get_id(), $)} CLASSES::Pseudogene->get(dbh=>$dbh, source=>"pseudogene.org");

while (my $gff_line = <GFF_FILE>) {
    next if (substr($gff_line, 0, 1) eq '#');
    chomp $gff_line;
    my @gff_arr = split(/\t/, $gff_line);

    my $fa_line = <FA_FILE>; chomp($fa_line);
    my $protein_align = <FA_FILE>; chomp($protein_align);
    my $pgene_trans = <FA_FILE>; chomp($pgene_trans);

    # Create protein
    my $protein_acc = $gff_arr[$gff_heading{query}];
    my $P = CLASSES::Protein->get_by_acc($dbh, $protein_acc);
    unless ($P) {
	print STDERR "Protein not found: $protein_acc\n";
	next;
    }

    # Create fragment
    $gff_arr[$gff_heading{chr}] =~ s/^chr//;
    my $chr = $gff_arr[$gff_heading{chr}];
    my $C = $build->get_chromosome($chr);
    my ($start_seg, $start_offset) = $C->get_segment_offset($gff_arr[$gff_heading{chrom_start}]);
    my ($finish_seg, $finish_offset) = $C->get_segment_offset($gff_arr[$gff_heading{chrom_end}]);
    
    my @frag_coords = ();
    my $str =  $gff_arr[$gff_heading{exon_bound}];
    $str =~ s/[\(\)]//g;
    my @exon_coords = split(/\s+/, $str);
    for (@exon_coords) {
	my ($s, $f) = /(\d+)..(\d+)/;
	push (@frag_coords, [$s, $f]);
    }
    
    my @F;
    for my $coords (@frag_coords) {
	my $F = undef;
	my @overlapping = CLASSES::Fragment->get_by_location($dbh, $C, $coords->[0], $coords->[1]);
	for (@overlapping) {
	    my ($s, $f) = $_->get_coords();
	    if ($s == $coords->[0] && $f == $coords->[1]) {
		$unused{$_->get_id()} = undef;
		$F = $_;
		last;
	    }
	}
	
	if (!$F) {
	    my ($start_seg, $start_offset) = $C->get_segment_offset($coords->[0]);
	    my ($finish_seg, $finish_offset) = $C->get_segment_offset($coords->[1]);
	    $F = CLASSES::Fragment->new(dbh => $dbh, 
					start_seg => $start_seg->get_id(), 
					start_offset => $start_offset, 
					finish_seg => $finish_seg->get_id(), 
					finish_offset => $finish_offset, 
					strand =>$gff_arr[$gff_heading{strand}]);
	}
	
	push(@F, $F);
    }
    
    my @new_frags = grep {!$_->get_id()} @F;

    CLASSES::Fragment->insert(@new_frags) if @new_frags > 0;
    
    # Add alignment
    # ADDITION: WILL NEED TO RECALCULATE THE ALIGNMENT IF PROTEIN SEQUECNES ARE DIFFERENT
    my $protein_seq = $protein_align;
    $protein_seq =~ s/[-|\/|\\]//g;
    
    my $A;
    my @A;
    my @align_frags = split(/\s+/, $protein_align);
    my @pgene_frags = split(/\s+/, $pgene_trans);
    assert(@align_frags==@F, "Bad number align frags\n");
    assert(@pgene_frags==@F, "Bad number pgene frags\n");
    for (my $i=0; $i < @F; $i++) { 
	my $F = $F[$i];
	$A = CLASSES::Sequence_Alignment->get_by_fragment_and_protein($dbh, $F, $P) ||  # id it dosn't find an alignmnet
	    CLASSES::Sequence_Alignment->new(dbh => $dbh,
					     protein_id => $P->get_id(),
					     frag_id => $F->get_id(),
					     tool => "Unknown",
					     params => "Unknown",
					     protein_start => $gff_arr[$gff_heading{query_start}],
					     protein_finish => $gff_arr[$gff_heading{query_end}],
					     fragment_start => 1,
					     fragment_finish => length($F->get_sequence())+1,
					     e_value => $gff_arr[$gff_heading{'expect'}],
					     aa_ident => $gff_arr[$gff_heading{ident}],
					     #dna_ident => $gff_arr[$gff_heading{AA_ident}],
					     protein_align_seq => $align_frags[$i],
					     frag_align_seq => $pgene_frags[$i]);
	push(@A, $A);
    }	      
    
    my @new_alignments = grep {!$_->get_id()} @A;

    CLASSES::Sequence_Alignment->insert(@new_alignments) if @new_alignments > 0;
    
#    next unless $gff_arr[$gff_heading{class_new}] =~ /FRAG|FP|PSSD1|PSSD2|DUP/;
    my $ptype = $gff_arr[$gff_heading{class_new}];
    if ($ptype eq 'PSSD1' || $ptype eq 'PSSD2') {
	$ptype = 'Processed';
    }
    elsif ($ptype eq 'DUP') {
	$ptype = 'Duplicated';
    }
    elsif ($ptype eq 'FRAG' || $ptype eq 'FP') {
	$ptype = 'Ambiguous';
    }
    else {
#	die("Bad ptype: $ptype\n");
    }
 
    # Add pgene
    my @pairs;
    for (my $i=0; $i < @F; $i++) {
	push(@pairs, [$F[$i], $A[$i]]);
    }
    
    $gff_arr[$gff_heading{comment}] = "" unless $gff_arr[$gff_heading{comment}];
    my $comment = $gff_arr[$gff_heading{comment}] || "";
    $comment = $gff_arr[$gff_heading{comment2}] ? $comment."   ".$gff_arr[$gff_heading{comment2}] : $comment;
    my $Pg = CLASSES::Pseudogene->new(dbh => $dbh || die("dbh\n"),
				      source => "pseudogene.org",
				      ptype => $ptype,
				      cytogenic_band => $gff_arr[$gff_heading{band} || die("Error 1\n")],
				      #completeness => $gff_arr[$gff_heading{Completeness} || die("Error 2\n")],
				      polya => $gff_arr[$gff_heading{polya} || die("Error 3\n")],
				      disable => ($gff_arr[$gff_heading{disable}] ne 'D') ? $gff_arr[$gff_heading{disable}] : 'h',
				      #GC_Pgene => $gff_arr[$gff_heading{GC_Pgene}] || die("Error 5\n"),
				      #GC_Isochore => $gff_arr[$gff_heading{GC_Isochore}] || die("Error 6\n"),
				      #kimura_dist => $gff_arr[$gff_heading{Kimura_Distance}] || "",
				      comment => $comment,
				      fragment_info => \@pairs);
    
    my ($s, $f) = $Pg->get_coords();
    my @overlapping = CLASSES::Pseudogene->get_by_location($dbh, $C, $s-100, $f+100);
    for my $Pg2 (@overlapping) {
	my @coords2 = $Pg2->get_coords();
	my $overlap_start = $coords2[0] > $s ? $coords2[0] : $s;
	my $overlap_finish = $coords2[1] < $f ? $coords2[1] : $f;
	my $overlap_length = $overlap_finish - $overlap_start;
	my $overlap1 = 1.0*$overlap_length / $Pg->get_length();
	my $overlap2 = 1.0*$overlap_length / $Pg2->get_length();
	if (($overlap1 >= 0.95 && $overlap2 >= 0.95) || ($Pg2->get_protein() && $P->get_acc() eq $Pg2->get_protein()->get_acc())) {
	    $Pg->set_acc($Pg2->get_acc());
	    last;
	}
    }
    
    # Make sure this isn't EXACTLY the same as some other pseudogene
    my $found = 0;
    for my $Pg2 (@overlapping) {
	my ($s2, $f2) = $Pg2->get_coords();
	if ($s==$s2 && $f==$f2 && $Pg->get_source() eq $Pg2->get_source() && $Pg->get_ptype() eq $Pg2->get_ptype() && $Pg->pgene_name() eq $Pg2->pgene_name()) {
	    my @f1 = $Pg->get_fragments();
	    my @f2 = $Pg->get_fragments();
	    if (@f1 == @f2) {
		$found = 1;
		for (my $i=0; $found && $i < @f1; $i++) {
		    $found = ($f1[$i]->get_id() == $f2[$i]->get_id());
		}
	    }

	    last if $found;
	}
    }

    if (!$found) {
	$Pg->insert();
    }
}

my @P = CLASSES::Pseudogene->get(dbh=>$dbh, where=>"source = 'pseudogene.org'");
if (@P > 0) {
    my $S_old = CLASSES::PFSet->get_by_acc($dbh, 2);
    my $S = CLASSES::PFSet->new(dbh => $dbh,
				acc => $S_old ? $S_old->get_acc() : undef,
				name => "Pseudogene.org pipeline output",
				notes => "Pseudogenes identified by the pseudogene.org pipeline",
				detailed_notes => "All pseudogenes identified by the Pseudogene.org pipeline, run on build $build_number",
				reference => "www.pseudogene.org", 
				ref_address => "http://www.pseudogene.org",
				original_set => $S_old ? $S_old->get_original_set() : undef,
				build => $build_number,
				pgene_list => \@P);
    $S->insert();
}

while (my ($id, $P) = each %unused) {
    CLASSES::Pseudogene->mark_dead($dbh, $P) if $P;
}

sub min {
    return $_[0] < $_[1] ? $_[0] : $_[1];
}

sub max {
    return $_[0] > $_[1] ? $_[0] : $_[1];
}
