#!/usr/bin/perl -w

use strict;
use DB::db;
use CLASSES::Pseudogene;

use Data::Dumper;

=header
create table pgene_all(
	pid int(10) not null,
	genome_ac int(10) not null,
	chr varchar(10),
	strand int(2),
	start int(12),
	stop int(12),
	dna text,
	primary key(pid, genome_ac)
);
=cut

my $dbh = DB::db::getDB(db => "Family");
my $sth = $dbh->prepare("select genome_ac, genome from genome_count");
my $sth3 = $dbh->prepare("insert into pgene_all values (?,?,?,?,?,?,?)");
my $sth4 = $dbh->prepare("select pid, genome_ac from pgene_all");

my %visited;
$sth4->execute();
while (my ($pid, $genome) = $sth4->fetchrow_array){
	$visited{$genome}{$pid} = 1;
}

my %genomes;
$sth->execute();
while (my ($genome_ac, $genome) = $sth->fetchrow_array){
	$genomes{$genome_ac} = $genome; 
}

foreach my $genome_ac(keys %genomes){
	next if $genome_ac != 9606;
	my $dbh2 = DB::db::getDB(db => "Pgenes_$genome_ac");
	my $sth5 = $dbh2->prepare("select id from Pseudogene A, Pseudogene_Version B where A.acc = B._acc and A.version = B._version and A.source = 'pseudogene.org'");
	$sth5->execute();
	while (my ($id) = $sth5->fetchrow_array){

		my $P = CLASSES::Pseudogene->get_pk($dbh2, $id);
		next if defined $visited{$genome_ac}{$id};
		my $chr = $P->get_chromosome()->get_number();
		my ($start, $stop) = $P->get_coords();
		my $strand = $P->get_strand();
		my $dna = $P->get_sequence();
		print ">$genome_ac ID:$id CHR:$chr POS:$strand($start, $stop)\n";
		print "$dna\n"; 
		$dna = "" if length($dna) > 100000;
		$sth3->execute($id, $genome_ac, $chr, $strand, $start, $stop, $dna);
	}	
	$dbh2->disconnect();
} 
