#!/usr/bin/perl

use strict;
use DB::db;
use Data::Dumper;
use CLASSES::Chromosome;
use CLASSES::Pseudogene;
use CLASSES::Build;

my $dbh = DB::db::getDB(db => "Family");
my $sth = $dbh->prepare("select genome_ac from genome_count");
my $sth2 = $dbh->prepare("select cluster_ac, A.seq_ac, seq_name, genome_ac, chromosome, complement, B.start, B.stop from hmm_clustering A, complete_genome B where A.seq_ac = B.seq_ac");
my $sth3 = $dbh->prepare("select A.genome_ac, pgene, protein, dna from current_pgene_protein A, pgene_all B where A.genome_ac = B.genome_ac and A.pgene = B.pid");


my %genome_dbh;
my %builds;

$sth->execute();
while (my ($genome_ac) = $sth->fetchrow_array){
#	print "$genome_ac\n";
	my $dbh_genome = DB::db::getDB(db => "Pgenes_$genome_ac");	
	$genome_dbh{$genome_ac} = $dbh_genome; 
	my $build = CLASSES::Build->current_build_number($dbh_genome);
	$builds{$genome_ac} = $build;	
}

my %fams;
$sth2->execute();
while (my ($cluster, $seq_ac, $seq_name, $genome, $chr, $complement, $start, $stop) = $sth2->fetchrow_array){
	my $strand;
	$strand = "+" if $complement == 1;
	$strand = "-" if $complement == -1;
	$fams{$cluster}{$seq_ac} = [$seq_name, $genome, $chr, $strand, $start, $stop]; 
}

my %pgenes;
$sth3->execute();
while (my ($genome, $pid, $protein, $dna) = $sth3->fetchrow_array){
	push @{$pgenes{$protein}}, [$genome, $pid, $dna];  
}

foreach my $cluster(sort {$a <=> $b} keys %fams){
	print "FAM:$cluster\n";
	open (OUT, ">/mnt/disk0/FAM_SEQS/$cluster") or die;
	foreach my $seq(keys %{$fams{$cluster}}){
		my $seq_name = $fams{$cluster}{$seq}->[0];
		my $genome = $fams{$cluster}{$seq}->[1];
		my $chr = $fams{$cluster}{$seq}->[2];
		my $strand = $fams{$cluster}{$seq}->[3];
		my $start = $fams{$cluster}{$seq}->[4];
		my $stop = $fams{$cluster}{$seq}->[5];

		my $chr_obj = CLASSES::Chromosome->get_chromosome($genome_dbh{$genome}, $builds{$genome}, $chr);
		if (defined $chr_obj and defined $start and defined $strand){	
			my $pro_dna = $chr_obj->get_sequence($start, $stop, $strand);
			print OUT ">$seq-$genome $seq_name\n";
			print OUT "$pro_dna\n";
		}
		next if not defined $pgenes{$seq_name};
		my @pseudos = @{$pgenes{$seq_name}};
		foreach my $p(@pseudos){
			my $genome2 = $p->[0];
			my $id = $p->[1];
			my $dna = $p->[2];
			
			print OUT ">p$id-$genome2 pseudogene\n";
			print OUT "$dna\n";
		}

	}
	close OUT;
}
