#!/usr/bin/perl -w
use strict;

#=============================================================
# Display functional gene information
#==============================================================
use DB::db;
use CLASSES::Pseudogene;
use SEARCH::Search;
use QUERY::query_factory;
use TaxID::tax_id;
use CGI qw(:all);
use Data::Dumper;

my $command = "group by Pseudogene.id";

my $output_type = param("output"); # || 'html';
my $sort = param("sort"); # || 0;
my $all = param("all"); # || 1;
my $set_id = param("set_search"); # || 53;
my $all_versions = param("all_versions") || $set_id;

print $output_type eq 'html' ? header() : header("text/plain");

my $str = "";     # $str will hold the entire text, and be printed at the end
my $file = undef;    # file name to check / create for a cashed result (only relevan when $all defined

eval {

my $tax_id = param("tax_id") || 9606;
my $org = get_org($tax_id);

if (not defined $set_id){
	$set_id = 1;
	$set_id = 53 if $tax_id == 9606;
	$set_id = 2 if $tax_id == 9598;
	$set_id = 2 if $tax_id == 10090;
	$set_id = 2 if $tax_id == 7227;
}

my @criterion;
my @operator;
my @searchValue;
my @booleans;

my $i = 0;

while (my $c = param("criterion$i")) {
    push(@criterion, $c);
    push(@operator, param("operator$i"));
    push(@searchValue, param("searchValue$i\_0"));
    push(@booleans, param("andOr$i"));
    $i++;
}

my $dbh = DB::db::getDB(db => "Pgenes_$tax_id");
my $S = SEARCH::Search->new($dbh);

my %sets;
my $sth2 = $dbh->prepare("select id, acc from PFSet"); 
$sth2->execute();
while (my ($id, $acc) = $sth2->fetchrow_array){
	$sets{$acc} = $id if not defined $sets{$acc};
	$sets{$acc} = $id if defined $sets{$acc} and $id > $sets{$acc};
}

my $dbh2 = DB::db::getDB(db => "Family");
my $sth = $dbh2->prepare("select cluster_ac, seq_name, pfam_id from complete_genome A, hmm_clustering B where A.seq_ac = B.seq_ac");
my %pfams;
my %pfam_address;
$sth->execute();
while (my ($fam, $seq, $pfam) = $sth->fetchrow_array){
	#print "$seq -> $pfam\n";
	$pfam_address{$seq} .= " <a href=\"http://www.pseudogene.org/FAMILY/family_seq_show.php?cluster_ac=$fam\">$fam</a>";
	next if not defined $pfam or $pfam eq "";
	my $pfam_place; 
	if ($pfam =~ /^(\w+)\./){$pfam_place = $1;}
	next if defined $pfams{$seq}{$pfam_place}; 
	$pfams{$seq}{$pfam_place} = 1;
	my $html_address = "http://www.sanger.ac.uk/cgi-bin/Pfam/getacc?$pfam_place";
	my $pfam_string = "(<A HREF=$html_address>$pfam_place</A>)";
	$pfam_address{$seq} .= $pfam_string;
}

my %extensions;

if ($criterion[0] && $criterion[0] =~ /search_extended\_(.+)/) {
    $extensions{$1} = 1;
}

for (CLASSES::Pseudogene->get_db_extensions($dbh)) {
    $extensions{$_} = 1 if param($_);
}

#print "<BR><CENTER>WARNING: Page is currently being debugged -- may not work.<BR>Will be back up in a few minutes.</CENTER><BR><BR>";

my @extensions = keys %extensions;

my @results;
my $set_id_str = undef;
if ($set_id) {
    my $query = create_query(type => "SVSelect",
			     dbh  => $dbh,
			     table => "PFSet_to_Pgene",
			     return_field => "_Pgene_id",
			     where => "_PFSet_id = $set_id");
    $query->execute();
    $set_id_str = join(" or ", map {"Pseudogene.id=$_"} $query->fetch_values());
}
    
if ($all) {
    if ($set_id) {
	if (@extensions == 0) {
	    $file = TOOLS::constants::CACHE_DIR."/$tax_id.$set_id.$output_type";

	    if (-e $file) {              # Cached file exists -- print it  (is there a better way???)
		open(IFILE, $file);
		for (<IFILE>) {
		    print $_;
		}
		close(IFILE);
		exit(0);
	    }
	}
	@results = CLASSES::Pseudogene->get(dbh=>$dbh, join_tables=>["PFSet_to_Pgene"], where=>"PFSet_to_Pgene._Pgene_ID=Pseudogene.id and PFSet_to_Pgene._PFSet_ID=$set_id", all_versions=>$all_versions, command=>$command);
    }
    else {
	@results = CLASSES::Pseudogene->get(dbh=>$dbh, all_versions=>$all_versions, command=>$command);
    }
}
else {
    my @clauses;

    push(@clauses, $S->search($criterion[0], $searchValue[0], $operator[0], undef, $all_versions));

    for (my $i=1; $i < @criterion; $i++) {
	my $component = $S->search($criterion[$i], $searchValue[$i], $operator[$i], undef, $all_versions);
	if ($booleans[$i-1] eq "and") {
	    $clauses[-1] = combine_component($clauses[-1], $component, "and");
	}
	else {
	    push(@clauses, $component);
	}
    }

    my $C = $clauses[0];
    $C || die("Problem in clause calculation");
    for (my $i=1; $i < @clauses; $i++) {
	$C = combine_component($C, $clauses[$i], "or");
    }
    if ($set_id) {
	my $where = QUERY::Where->new(clauses=>["PFSet_to_Pgene._Pgene_ID=Pseudogene.id", "PFSet_to_Pgene._PFSet_ID=$set_id"]);
	$C = combine_component($C, [["PFSet_to_Pgene"], $where, {}], "and");
    }
    @results = CLASSES::Pseudogene->get(dbh=>$dbh, join_tables => $C->[0], where => $C->[1], values => $C->[2], all_versions=>$all_versions, command=>$command);
}

#### Sort
@results = sort {$a->get_acc() <=> $b->get_acc()} @results if $sort == 0;
@results = sort cmp_pgenes @results if $sort == 1;
@results = sort cmp_protein @results if $sort == 2;   

###### Get all extension names (if necessary)
if ($output_type eq 'html' || $output_type eq 'text') {
my $length = @results;
if ($length == 0 ) { 
    if ($output_type eq 'html') {
	$str = $str.'<CENTER><IMG src="/pgtop.jpg"></CENTER><BR>';

	$str = $str."<FONT SIZE='4pt' CLASS='sans-serif' FACE='arial, Helvetica'>";
	$str = $str."<p>No pseudogenes meeting criteria of search found";
	$str = $str."".($set_id ? " in selected set" : " in current database").".<p>";
	#print "<p><center><A href='./pseudo.cgi'>Go back</A><p><html>";
    }
    else {
	$str = $str."";    # Why is this here???
    }
}
else {
    if ($output_type eq 'html') {
	$str = $str.'<CENTER><IMG src="/pgtop.jpg"></CENTER><BR>';

	$str = $str."<TITLE>Search Results ($org)</TITLE>".
	      "<link href=\"/pg.css\" rel=stylesheet\" type=\"text/css\">".
	       "<BODY>";
	$str = $str."<FONT SIZE='-1' color ='green' CLASS='sans-serif' FACE='arial, Helvetica'>".
	"<IMG border ='0' src = '../human/reddot.gif'>Click on the red button to view nucleotide sequence.<BR>".
	"<IMG border ='0' src = '../human/yellowdot.gif'>Click on the yellow button to view amino acid sequence.<BR>".
	"<IMG border ='0' src = '../human/bluedot.gif'>Click on the blue button to view the UCSC browser.</font><BR>".	
	"<FONT SIZE='-1' color='green' CLASS='sans-serif' FACE='arial, Helvetica'>Click on the Pseudogene Accession Number for details on the pseudogene.".	
	"</FONT></center><hr>";	
    }

    my @attributes = ("Links", "Pseudogene Accession Number (lsid format)", "Name", "Chromosome", "Start", "Stop", "Strand", "Type", "Protein Acc", "Protein Name", "Gene ID", "Source", "Annotation", "PseudoFAM(Pfam)", "Layer", @extensions);
    if ($output_type eq 'html') {
	$str = $str."<TABLE align='center' CELLPADDING='2' WIDTH='1200' CELLSPACING='1' BORDER='1'><TR align='center' BGCOLOR='#e3fdfa'>";
    
	# print the table header
	foreach my $aa (@attributes) {
	    $str = $str."<td><B><font size='-1' face=\"sans-serif,helvetica,arial\">$aa</font></td>\n";
	}
    }
    else {
	$str = $str.join("\t", @attributes)."\n";
    }
   
    for my $P (@results) {
	my $id = $P->get_id();
	my $acc = $P->get_lsid();
	my $name = $P->pgene_name();
	my $chr_num = $P->get_chromosome()->get_number();
	my ($start, $stop) = $P->get_coords();
	my $strand = $P->get_strand();
	my $type = $P->get_ptype();
	my $source = $P->get_source();
	my $ucsc_link = $P->get_ucsc_link();
	my $annotation = $P->get_annotation() || "";
	my $layer = $P->get_layer() || "";
	my $layer_info = "";

	if ($layer =~ /(\d+)\((\d+)\)/){
		my $thing = $1;
		my $set_acc = $2;
		my $set_id = $sets{$set_acc};
		$layer_info = "Level $thing (set <a href=\"http://www.pseudogene.org/cgi-bin/search.cgi?tax_id=$tax_id&search_type=Pseudogene&set_search=$set_id\">$set_acc</a>)";
	}

	my ($protein_acc, $protein_name, $dna, $aa, $gene_id, $p_link) = ("", "", "", "", "", "");

	my $pfam_belong;

	if ($P->get_protein()) {
	    $protein_acc = $P->get_protein()->get_acc();
	    $protein_name = $P->get_protein()->get_name() || "";
	    $pfam_belong = $pfam_address{$protein_acc};	
	    
	    $gene_id = $P->get_protein()->get_gene_id() || "";
	    $p_link = $P->get_protein()->get_link();
	}
	else {
	    ($protein_acc, $protein_name) = ("unknown", "unknown");
	}

	$pfam_belong = "" if not defined $pfam_belong;

	$dna = $P->get_fragment_sequence();
	$aa = $P->get_aligned_translation();

	my @extension_values = map {$P->get_extension_attributes($_)} @extensions;
	@extension_values = map {defined $_ ? $_ : ""} @extension_values;

	if ($output_type eq 'html') {
	    $str = $str."<TR align='center' BGCOLOR='#F3F3F3'>";

	    # display the links to sequence information
	    
	    $str = $str."<TD><A href='dna_pr_display.cgi?c_type=nucleotide&acc=$acc&seq=$dna' target='_new'>".
	    "<img border='0' src='../human/reddot.gif'></A>".
	    "<A href='dna_pr_display.cgi?c_type=amino_acid&acc=$acc&seq=$aa' target='_new'>".
	    "<img width='12' border='0' src='../human/yellowdot.gif'></A>".
	    "<A href='$ucsc_link' target='_new'>".
	    "<img width='12' border='0' src='../human/bluedot.gif'></A></TD>";	    


	    # display the table 
	    $str = $str."<TD><font size='-1'><A href='display-by-acc.cgi?internal_id=$id&tax_id=$tax_id' target='_new'>$acc</font></TD>".
	    "<TD><font size='-1'>$name</font></TD>".
	    "<TD><font size='-1'>".($ucsc_link ? "<a href='$ucsc_link'>$chr_num</a>" : "$chr_num")."</font></TD>".
	    "<TD><font size='-1'>$start</font></TD>".
	    "<TD><font size='-1'>$stop</font></TD>".
	    "<TD><font size='-1'>$strand</font></TD>".
	    "<TD><font size='-1'>$type</font></TD>".
	    "<TD><font size='-1'>".($p_link ? "<a href = '$p_link'>$protein_acc</a>" : "$protein_acc")."</font></TD>".
	    "<TD><font size='-1'>$protein_name</font></TD>".
	    "<TD><font size='-1'>$gene_id</font></TD>".
	    "<TD><font size='-1'>$source</font></TD>".
	    "<TD><font size='-1'>$annotation</font></TD>".
	    "<TD><font size='-1'>$pfam_belong</font></TD>".
	    "<TD><font size='-1'>$layer_info</font></TD>";


	    $str = $str.join(" ", map {"<TD><font size='-1'>$_</font></TD>"} @extension_values);

	    # "<TD><table></tr>";
	    # print "</TABLE></TD>",
	    $str = $str."</TR>";
	}
	else {
	    $str = $str.join("\t", ($name, $chr_num, $start, $stop, $strand, $type, $protein_acc, $protein_name, $gene_id, $source, @extension_values))."\n";
	}
    }
    $str = $str."</table>" if $output_type eq 'html';    
}

#$str = $str."<FONT SIZE='3pt' CLASS='sans-serif' FACE='arial, Helvetica'>",
#    "<p><p><center><A href=\"javascript:PopUp('../human/help_result.html', 800, 600, 60, 60)\" ONCLICK=\"PopUp('../human/help_report.html', 1000, 600, 60, 60); return true;\">Help</A> for this page</center>",
#    "</FONT></body></html>";
}
else {    # $output_type eq 'gtf'
    for (@results) {
	$str = $str.$_->gtf_string();
    }
}

print $str;
if ($file) {
    open(OFILE, ">$file") || die("Can't open file: $file");
    print OFILE $str;
    close(OFILE);
}
};

if ($@) {
   my $error = $@;
#   $error =~ s/\|,*$//;
#   $error =~ /^(.*)\sat\s/;
#   $error = $1;
   if ($output_type eq 'html') {
       $str = $str."<BR><BR>";
       $str = $str."<FONT SIZE='4' color='red' CLASS='sans-serif' FACE='arial,helvetica'>ERROR: $error</FONT><BR>";
       $str = $str.end_html;
   }
   else {
       $str = $str."ERROR: $error\n";
   }

   print $str;
}

sub combine_component {
    my ($c1, $c2, $bool) = @_;
    return [[@{$c1->[0]}, @{$c2->[0]}], QUERY::Where->new(clauses=>[$c1->[1], $c2->[1]], binary_op => $bool), {%{$c1->[2]}, %{$c2->[2]}}];
}

sub cmp_pgenes {
    my ($chr1, $chr2) = map {$_->get_chromosome()->get_number()} ($a, $b);

    if ($chr1 ne $chr2) {
	my $isNum1 = ($chr1 =~ /^\d+$/);
	my $isNum2 = ($chr2 =~ /^\d+$/);

	return $chr1 <=> $chr2 if $isNum1 && $isNum2;
	return $chr1 cmp $chr2 unless $isNum1 || $isNum2;
	return -1 if $isNum1;
	return 1;
    }

    my @c1 = $a->get_coords();
    my @c2 = $b->get_coords();
    return ($c1[0] <=> $c2[0] || $c1[1] <=> $c2[1]);
}


sub cmp_protein {
    my ($P1, $P2) = map {$_->get_protein()} ($a, $b);
    return 0 unless $P1 || $P2;
    return -1 unless $P2;
    return 1 unless $P1;
    return $P1->get_acc() cmp $P2->get_acc();
}
