#!/usr/bin/perl -w
use strict;

#=============================================================
# Display functional gene information
#==============================================================
use DB::db;
use CLASSES::Pseudogene;
use SEARCH::Search;
use QUERY::query_factory;
use TaxID::tax_id;
use CGI qw(:all);
use Data::Dumper;
use UNION;
use javascript::TRANSFER;


my $output_type = param("output") || 'html';
my $sort = param("sort");
my $all = param("all");

print $output_type eq 'html' ? header() : header("text/plain");

eval {

my $tax_id = param("tax_id") || 9606;
my $org = get_org($tax_id);

my @criterion;
my @operator;
my @searchValue;
my @booleans;

my $i = 0;
while (my $c = param("criterion$i")) {
    push(@criterion, $c);
    push(@operator, param("operator$i"));
    push(@searchValue, param("searchValue$i\_0"));
    push(@booleans, param("andOr$i"));
    $i++;
}

my $dbh = DB::db::getDB(db => "Pgenes_$tax_id");
my $S = SEARCH::Search->new($dbh);

my @extensions = ();

if ($criterion[0] =~ /search_extended\_(.+)/) {
    push(@extensions, $1);
}

my  @results;

if ($all) {
    @results = CLASSES::PFSet->get(dbh=>$dbh);
}

else {
    @results = $S->search($criterion[0], $searchValue[0], $operator[0]);
    
    for (my $i=1; $i < @criterion; $i++) {
	if ($criterion[$i] =~ /search_extended\_(.+)/) {
	    push(@extensions, $1);
	}

	if ($booleans[$i-1] eq 'and') {
	    my $merge_string = join(" or ", map {"PFSet.id=".$_->get_id()} @results);
	    @results = $S->search($criterion[$i], $searchValue[$i], $operator[$i], $merge_string);
	}
	else {
	    my @results2 = $S->search($criterion[$i], $searchValue[$i], $operator[$i]);
	    my %H;
	    foreach (@results) {
		$H{$_->get_id()} = $_;
	    }
	    foreach (@results2) {
		$H{$_->get_id()} = $_;
	    }
	    @results = values %H;
	}
    }
}

#### Sort
@results = sort {$a->get_acc() <=> $b->get_acc();} @results if $sort == 0;
@results = sort {$a->get_name() cmp $b->get_name();}  @results if $sort == 1;
@results = sort {$a->get_notes() cmp $b->get_notes();} @results if $sort == 2;   

if ($output_type eq 'html' || $output_type eq 'text') {
my $length = @results;
if ($length == 0 ) { 
    if ($output_type eq 'html') {
	print "<FONT SIZE='4pt' CLASS='sans-serif' FACE='arial, Helvetica'>";
	print "<p>No sets found.<p>";
	print "<p><center><A href='./pseudo.cgi'>Go back</A><p><html>";
    }
    else {
	print "No sets found.";
    }
}
else {

    if ($output_type eq 'html') {
	print "<html>";
	print "<head>";
	print "<meta http-equiv=\"Content-Language\" content=\"DE\">\n";
	print "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n";
	print "<meta http-equiv=\"Content-Style-Type\" content=\"text/css\">\n";
	print '<link rel="stylesheet" href="/style.css">';
	print '<script language="JavaScript" type="text/javascript" src="/infobox.js"></script>';
	print "</head>";
	print "<body>";
	print '<CENTER><IMG src="/pgtop.jpg"></CENTER><BR>';
	print "<div id=\"infodiv\" style=\"position: absolute; visibility: hidden; z-index: 20; top: 0px; left: 0px;\"></div>";
 
	print "<h2 align=center>Pseudogene Sets</h2>"; 
	print "<script language=\"JavaScript\" type=\"text/javascript\">\n";
	print "<!--\n";	
	my $count1 = 0;
	for my $S(@results){
		my $id = $S->get_lsid();
		my $dataSource = $S ? $S->get_detailed_notes() : "";
		$dataSource = "NO DESCRIPTION" if not defined $dataSource; 
	        print "maketip('detailed_notes$count1', '$id','$dataSource')\n";
		$count1++;	
	}
	print "// -->\n</script>";

	my $count2 = 0;
	for my $S(@results){
		my $id = $S->get_lsid();
		my $dataSource = $S ? $S->get_detailed_notes() : "";
		$dataSource = "NO DESCRIPTION" if not defined $dataSource;
	    	print "<div id=\"detailed_notes$count2\" name=\"detailed_notes$count2\" style=\"position: absolute; visibility: hidden; z-index: 20; top: 0px; left: 0px;\"><table bgcolor=\"#333399\" border=\"0\" cellpadding=\"2\" cellspacing=\"0\" width=\"250\"><tbody><tr><td class=\"tiptd\"><table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\"><tbody><tr><th><span class=\"ptt\"><b><font color=\"#ffffff\">$id</font></b></span></th></tr></tbody></table><table bgcolor=\"#ccccff\" border=\"0\" cellpadding=\"2\" cellspacing=\"0\" width=\"100%\"><tbody><tr><td><span class=\"pst\"><font color=\"#000000\">$dataSource</font></span></td></tr></tbody></table></td></tr></tbody></table></div>\n";
		$count2++;
	}
    }


    my @attributes = ("Set Number", "Name", "Notes and Links", "Reference", "Current Build", "Current Size", "Original Build", "Original Size");
    if ($output_type eq 'html') {
	print "<TABLE align='center' CELLPADDING='2' WIDTH='800' CELLSPACING='1' BORDER='1'><TR align='center' BGCOLOR='#e3fdfa'>";
    
	# print the table header
	foreach my $aa (@attributes) {
	    print "<td><B><font size='-1' face=\"sans-serif,helvetica,arial\">$aa</font></td>\n";
	}
    }
    else {
	print join("\t", @attributes)."\n";
    }
	
	my @set_id_array = map {$_->get_acc()} @results;
	my @set_name_array = map {$_->get_name()} @results; 
	@set_name_array = map {"\"$_\""} @set_name_array;
	my $SetBox_str = &pass_array("setArray", \@set_id_array);
	my $SetName_str = &pass_array("setArrayName", \@set_name_array); 
	my $sort_string = '<TD> <INPUT TYPE="radio" NAME="sort" VALUE="0" checked> Accession</TD><TD> <INPUT TYPE="radio" NAME="sort" VALUE="1"> Location </TD> <TD> <INPUT TYPE="radio" NAME="sort" VALUE="2"> Protein </TD>';
    	my $print_string = '<TD><INPUT TYPE="radio" NAME="output" VALUE="html" CHECKED> html output &nbsp</TD><TD><INPUT TYPE="radio" NAME="output" VALUE="text"> text output &nbsp   </TD><TD><INPUT TYPE="radio" NAME="output" VALUE="gtf"> gtf output    </TD>';



	my $count = 0; 
    for my $S (@results) {

	my $id = $S->get_id();
	#my $acc = $S->get_lsid();
	my $acc = $S->get_acc();
	my $name = $S->get_name();
	my $notes = $S->get_notes();
	my $ref = $S->get_reference();
	my $ref_address = $S->get_ref_address();
	my $size = $S->get_size();
	my $build = $S->get_build();
	my $S2 = $S->get_original_set() ? CLASSES::PFSet->get_pk($dbh, $S->get_original_set()) : "";
	my $old_id = $S2 ? $S2->get_id() : "";
	my $old_size = $S2 ? $S2->get_size() : "";
	my $old_build = $S2 ? $S2->get_build() : "";
	my $dataSource = $S ? $S->get_detailed_notes() : "";
	my $link = $S ? $S->get_data_address() : ""; 

	$dataSource = "NO DESCRIPTION" if not defined $dataSource;

	if ($output_type eq 'html') {
	    print "<TR align='center' BGCOLOR='#F3F3F3'>";
	
	    # display the links to sequence information
	    
	    
	    # display the table 
	    # print "<TD><font size='-1'><A href='pgene.cgi?id=$id&tax_id=$tax_id&organism=$org' target='_new'>PGO.$tax_id.$acc</font></TD>",

	    print "<TD><font size='-1'><A href='/cgi-bin/search.cgi?tax_id=$tax_id&search_type=Pseudogene&set_search=$id'>$acc</a></font></TD>\n";
	    print "<TD align=\"LEFT\"><font size='-1'>$name</font></TD>\n";
	    #print "<TD align=\"LEFT\"><font size='-1'>$notes</font></TD>\n";

	    if (length($dataSource) > 40) {
		print "<TD><font size='-1'><A HREF=\"$link\" ONMOUSEOVER=\"tip('detailed_notes$count')\" ONMOUSEOUT=\"untip()\">"
		    . substr($dataSource, 0, 40) . "...</A></font></TD>\n";
	    } else {
		print "<TD><font size='-1'><A HREF=\"$link\" ONMOUSEOVER=\"tip('detailed_notes$count')\" ONMOUSEOUT=\"untip()\">"
                    . "$dataSource</A></font></TD>\n";
	    }
	    #print "<TD><font size='-1'><A HREF=\"$link\" ONMOUSEOVER=\"document.write('$dataSource');return true;\" ONMOUSEOUT=\"untip()\">contents</A></font></TD>\n";	

	    print "<TD align=\"LEFT\"><FONT size =-1>";
	    print "<A HREF=$ref_address>" if $ref_address;
	    print $ref;
	    print "</A>" if $ref_address;
	    print "</font></TD>\n";

	    print "<TD><font size='-1'>$build</font></TD>\n",
	    "<TD><font size='-1'>$size</font></TD>\n";

	    if ($S2) {
		print "<TD><font size='-1'><A href='/cgi-bin/search.cgi?tax_id=$tax_id&search_type=Pseudogene&set_search=$old_id'>$old_build</font></TD>\n",
		"<TD><font size='-1'>$old_size</TD>\n";
	    }
	    else {
		print "<TD></TD><TD></TD>";
	    }

	}
	else {
	    print join("\t", ($acc, $name, $notes, $ref, $build, $size, $old_build, $old_size)); 
	}
	$count++;
    }
    print "</table>" if $output_type eq 'html';    
    if ($output_type eq 'html') {
	my $sample_lsid = $results[0]->get_lsid();
	my $sample_acc = $results[0]->get_acc();
	$sample_lsid =~ s/$sample_acc$/X/;
	print "<CENTER><I>Accession Numbers follow the format: $sample_lsid where X is the Set Number</I></CENTER><BR>";
    }

	# 12/12/05 Modification start from here. Get the list data from ordered set list.

print "<br><br><br><hr size=5 color=black width=100% noshade>";
print "<br><b><font size=4>UNION SET CONSTRUCTION TOOL (choose at least two sets and input order is important)</font><br><br> ";
print <<HTML_BODY;
	<SCRIPT Language="JavaScript" Type="text/javascript">
	<!--
		// the array for new set
		var setArray = new Array();
		$SetBox_str;

		var setArrayName = new Array();
		$SetName_str;

                // The tbody element that contains the controls
                var tbObj;

                // Number of rows added
                var rowCount = 0;

                function addRow(criterionObj)
                {
                        if (document.getElementById)
                                tbObj = document.getElementById("searchTableBody");
                        else if (document.all)
                                tbObj = document.all["searchTableBody"];

/*                        if (criterionObj != null)
                        {
                                var criterionObjId = criterionObj.name.substring(9);
                                var andOrObj;
                                if (document.getElementById)
                                        andOrObj = document.getElementById("andOr" + criterionObjId);
                                else if (document.all)
                                        andOrObj = document.all["andOr" + criterionObjId];
                                else
                                        return;

                                var trObj = andOrObj.parentNode.parentNode;
                                if (trObj != tbObj.lastChild)
                                        return;
                        }
*/

                        // Create a new row and a new cell for the table
                        var newRowObj = tbObj.insertRow(-1);
                        var newCellObj = newRowObj.insertCell(-1);

			// Create the set input box
			var newSetBox = document.createElement("SELECT");
			newSetBox.id = "set" + rowCount;
			newSetBox.name = "set" + rowCount;
			newSetBox.onchange = function(){};
			var aaa = "Choose the " + (rowCount+1) + " set";
			var setIDString = document.createTextNode(aaa);
			newCellObj.appendChild(setIDString);

			for (i=0; i<setArray.length; i++){
				var bbb = "PFSet:" + setArray[i] + ": " + setArrayName[i];	
				newSetBox.options[i] = new Option(bbb, setArray[i]);

			}
			newCellObj.appendChild(newSetBox);

			if (rowCount > 0){
                        // And/Or radio buttons
                        var newAndObj = document.createElement("INPUT");
                        newAndObj.id = "andOr" + rowCount;
                        newAndObj.name = "andOr" + rowCount;
                        newAndObj.type = "radio";
                        newAndObj.value = "and";
                        // The checked statements in the following line are for handling a potential bug of IE
                        newAndObj.onclick = function() {newAndObj.checked = true; addRow(newSetBox);};
                        newCellObj.appendChild(newAndObj);
                        var newAndText = document.createTextNode(" more sets ");
                        newCellObj.appendChild(newAndText);
			}else{
				rowCount++;
				addRow(newSetBox);
			}

                        // Delete button
			if (rowCount > 2 ){	
                                var newDeleteObj = document.createElement("INPUT");
                                newDeleteObj.type = "button";
                                newDeleteObj.value = "Delete";
                                newDeleteObj.onclick = function() {deleteRow(newRowObj);};
                                newCellObj.appendChild(newDeleteObj);
			}
                        rowCount ++;
		}

                function deleteRow(rowObj)
                {
                        tbObj.deleteRow(rowObj.rowIndex);
                }


	//-->
	</SCRIPT>
	<FORM Name="searchForm" Action="join-set.cgi" Method="get">

		Pseudogene Overlap Definition: overlap more than <Input type="text" Name="overlap" value="0"> nucleotides.
		<INPUT Type="hidden" Name="tax_id" Value="$tax_id">
                <TABLE>
                <TBODY Id="searchTableBody">
                </TBODY>
                <TFOOT>
                <TR>
                        <TD><INPUT Type="submit" Value="Submit Sets"></TD>
                </TR>
                </TFOOT>
                </TABLE>
	
		<SCRIPT Language="JavaScript" Type="text/javascript">
		<!--
			addRow(null);
		//-->
		</SCRIPT>

		<TABLE WIDTH=658 BORDER=0 ALIGN=LEFT><TR><TD>
		<TABLE CLASS="table01" width=100%><TR><TD>
		<SPAN CLASS="title3"><b>Display Options</SPAN>
		    <TABLE>
		    <TBODY Id="searchOptions"> </TBODY>
		    <TFOOT>
		    <TR>	
		    <TR>
		    <TD> Sort by: </td>
		    $sort_string
		    </TR>
		    <TD>Output type: &nbsp </TD>
		    $print_string
		    </TR>
		    </TFOOT>
		    </TABLE>
		</TABLE>


HTML_BODY
print "</FORM>";
print "</body>";
print "</html>";
}

#print "<FONT SIZE='3pt' CLASS='sans-serif' FACE='arial, Helvetica'>",
#    "<p><p><center><A href=\"javascript:PopUp('../human/help_result.html', 800, 600, 60, 60)\" ONCLICK=\"PopUp('../human/help_report.html', 1000, 600, 60, 60); return true;\">Help</A> for this page</center>",
#    "</FONT></body></html>";
}
else {    # $output_type eq 'gtf'
    for (@results) {
	print $_->gtf_string();
    }
}

};

if ($@) {
   my $error = $@;
   $error =~ s/\|,*$//;
   $error =~ /^(.*)\sat\s/;
   $error = $1;
   if ($output_type eq 'html') {
       print "<BR><BR>";
       print "<FONT SIZE='4' color='red' CLASS='sans-serif' FACE='arial,helvetica'>ERROR: $error</FONT><BR>";
       print end_html;
   }
   else {
       print "ERROR: $error\n";
   }
}
sub merge_results {
    my @arr1 = @$_[0];
    my @arr2 = @$_[1];
    my $bool = $_[2];

    my %H;
    for (@arr1) {
	$H{$_} = 1;
    }

    for (@arr2) {
	$H{$_} = defined $H{$_} ? 2 : 1;
    }

    if ($bool eq '1') {
	return keys %H;
    }
    else {
	return grep {$H{$_}==2} keys %H;
    }
}

1;
