#!/usr/bin/perl -w 
use strict;

use CLASSES::Pseudogene;
use CLASSES::Protein;
use CLASSES::Sequence_Alignment;
use CLASSES::Build;
use CLASSES::Chromosome;
use ALIGNMENT::default_align;
use DB::db;


my $tax_id = 9606;
my $build_num = 34;
my $dbh = DB::db::getDB(db => "Pgenes_$tax_id");

my $B = CLASSES::Build->get_by_number($dbh, $build_num);
my %H;
open(IFILE, "tppg/psi-out.txt");
while (<IFILE>) {
   my ($old_name, $new_name, $acc) = split(/\t/, $_);
   $H{$old_name} = $acc;
   $H{$new_name} = $acc;
   my $line = <IFILE>;
}

open(IFILE, "tppg/tp-psi.coords");
while (<IFILE>) {
   chomp;
   my @arr = split(/\s+/, $_);
   $arr[4] =~ /ENS\:(\w+)\.(\d+)/;
   my ($chr, $shift) = ($1, $2);
   my ($start, $stop, $strand) = @arr[1,2,0];
   $start += $shift;
   $stop += $shift;
   $chr || die("Bad chromosome: $_\n");
   $arr[3] =~ /\_(.*)(\.\d+\_|\_\d+)$/ || die("Bad line: $_\n");
   my $pacc = $H{$1} || $1;

   my $P = undef;
   my $C = $B->get_chromosome($chr);
   my @overlaps = CLASSES::Pseudogene->get_by_location($dbh, $C, $start, $stop);
   for (@overlaps) {
      my @coords = $_->get_coords();
      if (abs($coords[0]-$start) < 12 && abs($coords[1]-$stop) < 12) {
         $P = $_;        
         last;
      }
   }

   if (!$P) {
       my $F = undef;
       my @overlapping = CLASSES::Fragment->get_by_location($dbh, $C, $start, $stop);
       for (@overlapping) {
	   my ($s, $f) = $_->get_coords();
	   if ($s == $start && $f == $stop && $_->get_strand() eq $strand) {
	       $F = $_;
	       last;
	   }
       }
	   
       if (!$F) {
	   my ($start_seg, $start_offset) = $C->get_segment_offset($start);
	   my ($finish_seg, $finish_offset) = $C->get_segment_offset($stop);
	   $F = CLASSES::Fragment->new(dbh => $dbh, start_seg => $start_seg->get_id(), start_offset => $start_offset, finish_seg => $finish_seg->get_id(), finish_offset => $finish_offset, strand =>$strand);
       }
	   
       $F->insert() unless $F->is_in_db(); 
	   
       ### Protein 
       my $Pro = CLASSES::Protein->get_by_acc($dbh, $pacc);

       ### Alignment
       my $A = undef;
       if ($Pro) {
	   $A = CLASSES::Sequence_Alignment->get_by_fragment_and_protein($dbh, $F, $Pro);
	   if (!$A) {
	       $A = default_align($Pro, $F, 100, 0);
	       $A->insert() if $A;
	   }
       }

       ### Pseudogene
       $P = CLASSES::Pseudogene->new(dbh => $dbh, 
				     source => "Hoppsigen", 
				     fragment_info => [[$F, $A]], 
				     ptype => "p");
       $P->insert();
   }

   print $P->get_id()."\t";
}
print "\n";
