package RELATIONS::Protein_ID;
our $VERSION = '1.0';
use strict;

use QUERY::Cache;
use QUERY::query_factory;
use TOOLS::assert;

our @ISA = qw(Exporter);
our @EXPORT = qw(get_all_secondary);

my $id_table = "Protein_ID";  # Name of reational table for looking up
                              # alternate ID names
#######################
# Name: get_primary_acc
# Purpose: Find the "primary" acc for a given acc
#  (e.g. When proteins have multiple accs between databases)
# Parameters: database handle, one or more secondary accs
# Output: An array (or single) of "primary" accs
# sub get_primary_acc {
#     my $dbh = assert_dbh(shift);
#     my @acc_arr = @_;
#     assert(wantarray || @acc_arr==1, "Cannot give multiple accs as parameters in scalar context\n");
#
#     my @results;
#     my $query = QUERY::Cache->get($dbh, $id_table, "get_primary_acc");
#     unless ($query) {
# 	$query = create_query("type"=>"SVSelect", 
# 			      "dbh"=>$dbh, 
# 			      "table"=>$id_table,
# 			      "return_field"=>"_acc", 
# 			      "where"=>"_secondary = ?_secondary?");
# 	QUERY::Cache->set($dbh, $id_table, "get_primary_acc", $query);
#     }
#     foreach (@acc_arr) {
# 	$query->execute("_secondary"=>$_);
# 	push(@results, $query->fetch_values());
#     }
#
#     return wantarray ? @results : $results[0];
# }

#######################
# Name: get_all_secondary
# Purpose: Find all secondaries mapping to a single primary
#  (e.g. When proteins have multiple accs between databases)
# Parameters: Database handle, one or more primary accs
# Output: Arrays of all secondary accs;
sub get_all_secondary {
    my $dbh = assert_dbh(shift);
    my @acc_arr = @_;

    my @results;
    my $query = QUERY::Cache->get($dbh, $id_table, "get_all_secondary");
    unless ($query) {
	$query = create_query("type"=>"SVSelect", 
			      "dbh"=>$dbh, 
			      "table"=>$id_table,
			      "return_field"=>"_secondary", 
			      "where"=>"_acc = ?_acc?");
	QUERY::Cache->set($dbh, $id_table, "get_all_secondary", $query);
    }
    foreach my $primary (@acc_arr) {
	$query->execute("_acc"=>$primary);
	foreach ($query->fetch_values()) {
	    push(@results, $_) if $_ ne $primary;
	}
    }

    return @results;
}

#######################
# Name: is_primary_acc
# Purpose: Determine if each acc is primary
# Parameters: One or more primary accs
# Output: Array of booleans in vector context
# sub is_primary_acc {
#     my $dbh = assert_dbh(shift);
#     my @acc_arr = @_;
#     assert(wantarray || @acc_arr<=1, "Cannot give multiple accs as parameters in scalar context\n");
#
#     my @results;
#     my $query = QUERY::Cache->get($dbh, $id_table, "is_primary_acc");
#     unless ($query) {
# 	$query = create_query("type"=>"SVSelect", 
# 			      "dbh"=>$dbh, 
# 			      "table"=>$id_table,
# 			      "return_field"=>"_acc", 
# 			      "where"=>"_acc = ?where_acc?");
# 	QUERY::Cache->set($dbh, $id_table, "is_primary_acc", $query);
#     }
#     foreach (@acc_arr) {
# 	$query->execute("where_acc"=>$_);
# 	push(@results, $query->fetch_values() > 0);
#     }
#
#     return wantarray ? @results : $results[0];
# }


#######################
# Name: is_secondary_acc
# Purpose: Determine if each acc is secondary
# Parameters: One or more accs
# Output: Array of booleans in vector contact
sub is_secondary_acc {
    my $dbh = assert_dbh(shift);
    my @acc_arr = @_;
    assert(wantarray || @acc_arr<=1, "Cannot give multiple accs as parameters in scalar context\n");

    my @results;
    my $query = QUERY::Cache->get($dbh, $id_table, "get_primary_acc");
    unless ($query) {
	$query = create_query("type"=>"SVSelect", 
			      "dbh"=>$dbh, 
			      "table"=>$id_table,
			      "return_field"=>"_acc", 
			      "where"=>"_secondary = ?_secondary?");
	QUERY::Cache->set($dbh, $id_table, "get_primary_acc", $query);
    }
    foreach (@acc_arr) {
	$query->execute("_secondary"=>$_);
	my @values = $query->fetch_values();
	push(@results, @values > 0 && $values[0] ne $_);
    }

    return wantarray ? @results : $results[0];
}
    

######################
# Name: add_secondaries
# Purpose: Add new secondary/primary acc pairs to the table
# Paramters: database handle (static version only), single primary, secondaries
# Output: None
# sub add_secondaries {
#     my $dbh = assert_dbh(shift);
#     my $primary = shift;
#     my @secondary = @_;

#     my @exists_as_secondary = is_secondary_acc($dbh, @secondary);
#     my @remaining;
#     for (my $i=0; $i < @secondary; $i++) {
# 	push(@remaining, $secondary[$i]) unless $exists_as_secondary[$i];
#     }

#     my $found = 0;
#     foreach (@remaining) {
# 	if ($_ eq $primary) {
# 	    $found = 1;
# 	    last;
# 	}
#     }
#     @remaining = ($primary, @remaining) unless ($found || is_primary_acc($dbh, $primary));
	
#     if (@remaining > 0) {
# 	my $query = QUERY::Cache->get($dbh, $id_table, "add_secondaries");
# 	unless ($query) {
# 	    $query = create_query("type"=>"Insert", 
# 				  "table"=>$id_table, 
# 				  "dbh"=>$dbh, 
# 				  "attributes"=>["_secondary","_acc"]);
# 	    QUERY::Cache->set($dbh, $id_table, "add_secondaries");
# 	}
# 	foreach (@remaining) {
# 	    $query->execute("_secondary"=>$_, "_acc"=>$primary);
# 	}
#     }
# }
