package genome;
$VERSION = 1.00;
use strict;
use integer;

# For quickly reading substrings in form large files in FASTA format.
sub new {
    my ($class, $file, $file2, $seg_length) = @_;
    die("Bad genome->new input\n") if $file=~/.gnm$/;
    $file2 = $file.".gnm" unless defined($file2);

    my $self = {};
    open(IFILE, $file2) || return genome->create($file, $file2, $seg_length);

    my $str;
    my @old_time;
    my @new_time;

    my ($write_time, $mod_time);
    chomp($write_time = <IFILE>);

    chomp($self->{"_size"} = <IFILE>);
    chomp($self->{"_seg_length"} = <IFILE>);
    chomp($self->{"_offset"} = <IFILE>);
    chomp($self->{"_file_name"} = <IFILE>);

    $mod_time = (stat $self->{"_file_name"})[9];

    if ($write_time <= $mod_time) {
	close(IFILE);
	my $genome = genome->create($self->{"_file_name"}, $self->{"_seg_length"});
	$genome->save($file2);
	return $genome;
    }

    chomp($str = <IFILE>);
    my @arr = split(/\s/, $str);
    $self->{"_arr"} = \@arr;
    
    my $fh;
    open($fh, $self->{"_file_name"});
    $self->{"_file_handle"} = $fh;

    return bless $self, $class;
}

sub create {
    my ($class, $file1, $file2) = @_;
    $file2 = "$file1.gnm" unless defined($file2);

    my $seg_length = (@_==4 && defined($_[3])) ? $_[3] : 50000;

    my @arr;
    my $fh;

    open($fh, $file1) || die("Genome->new: File not found: $file1\n");
    <$fh>;
    my $start = tell $fh;
    my $offset = $start;

    my $str;
    my $length = 0;
    while (read($fh, $str, $seg_length)) {
	if ($str =~ s/^(\s+)//) {
	    $start += length($1);
	}
	$str =~ s/\s//g;
   	my $moved = length($str);
	$length += length($str);
	while ($moved < $seg_length) {
	    read($fh, $str, $seg_length - $moved) || last;
	    $str =~ s/\s//g;
	    $moved += length($str);
	    $length += length($str);
	}
	push(@arr, $start) if $moved > 0;
	$start = tell $fh;
    }

    my $Hash = {"_file_handle" => $fh, "_arr" => \@arr, "_size" => $length, "_seg_length" => $seg_length, "_offset" => $offset, "_file_name" => $file1};
    (bless $Hash, $class)->save($file2);
}

sub get_sequence {
    my ($self, $start, $len) = @_;
    my $whence = (@_==4 && $_[3]==0) ? 0 : 1;

    $start -= $whence;
    ($start < $self->{"_size"}) || return undef;
    ($len > 0) || return undef;
    ($start >= 0) || return undef;
    
    my $fh = $self->{"_file_handle"};
    my $index = $start / $self->{"_seg_length"};
    my $cur_char = ($self->{"_seg_length"}*$index);

    my $byte = $self->{"_arr"}->[$index];
    seek $fh, $byte, 0;
    while ($cur_char < $start) {
	my $tmp;
	read $fh, $tmp, $start-$cur_char;
	$tmp =~ s/\s//g;
	$cur_char += length($tmp);
    }
    
    my $str;
    read $fh, $str, $len;
    $str =~ s/\s//g;
    while (length($str) < $len) {
	my $temp;
	read($fh, $temp, $len-length($str)) || last;
	$temp =~ s/\s//g;
	$str = $str.$temp;
    }

    return $str;
}

sub get_reverse_complement {
    my ($self, @args) = @_;
    my $str = $self->get_sequence(@args);

    $str =~ tr/acgtACGT/tgcaTGCA/;
    return reverse($str);
}

sub get_size {
    return $_[0]->{"_size"};
}

sub get_seg_length {
    return $_[0]->{"_seg_length"};
}

sub save {
    my ($self, $file) = @_;

    $file = $self->{"_file_name"}."gnm" if !defined($file);

    my $fh = $self->{"_file_handle"};

    open(OFILE, ">$file") || die("Genome: Can't save to $file.\n");
    print OFILE time."\n";
    print OFILE $self->{"_size"}."\n";
    print OFILE $self->{"_seg_length"}."\n";
    print OFILE $self->{"_offset"}."\n";
    print OFILE $self->{"_file_name"}."\n";
    print OFILE join(" ", @{$self->{"_arr"}})."\n";

    return $self;
}

sub DESTROY {
    my ($self) = @_;
    my $fh = $self->{"_file_handle"};
    close($fh);
}
